@@ -2,7 +2,7 @@
  This file is part of SLRN.
 
  Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
- Copyright (c) 2001 Thomas Schultz <tststs@gmx.de>
+ Copyright (c) 2001, 2002 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -15,8 +15,8 @@
  more details.
 
  You should have received a copy of the GNU General Public License along
- with this program; if not, write to the Free Software Foundation, Inc., 675
- Mass Ave, Cambridge, MA 02139, USA. 
+ with this program; if not, write to the Free Software Foundation, Inc.,
+ 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 #ifndef _SLRN_ART_H
 #define _SLRN_ART_H
@@ -29,7 +29,9 @@ extern SLKeyMap_List_Type *Slrn_Article_Keymap;
 extern char *Slrn_Quote_String;
 extern char *Slrn_Save_Directory;
 extern char *Slrn_Header_Help_Line;
+extern char *Slrn_Header_Status_Line;
 extern char *Slrn_Art_Help_Line;
+extern char *Slrn_Art_Status_Line;
 extern char *Slrn_Followup_Custom_Headers;
 extern char *Slrn_Reply_Custom_Headers;
 extern char *Slrn_Supersedes_Custom_Headers;
@@ -37,8 +39,6 @@ extern char *Slrn_Overview_Date_Format;
 extern char *Slrn_Followup_Date_Format;
 extern int Slrn_Use_Localtime;
 
-extern int Slrn_Simulate_Graphic_Chars;
-
 extern int Slrn_Article_Window_Border;
 extern int Slrn_Startup_With_Article;
 extern int Slrn_Show_Thread_Subject;
@@ -47,8 +47,6 @@ extern int Slrn_Query_Next_Group;
 extern int Slrn_Auto_CC_To_Poster;
 extern int Slrn_Score_After_XOver;
 extern int Slrn_Use_Tmpdir;
-extern int Slrn_Sorting_Mode;
-extern int Slrn_Threads_Visible;
 extern int Slrn_Wrap_Mode;
 extern int Slrn_Wrap_Method;
 extern int Slrn_Use_Header_Numbers;
@@ -59,6 +57,7 @@ extern int Slrn_Kill_Score_Max;
 extern FILE *Slrn_Kill_Log_FP;
 extern int Slrn_Signature_Hidden;
 extern int Slrn_Pgp_Signature_Hidden;
+extern int Slrn_Quotes_Hidden_Mode;
 extern int Slrn_Verbatim_Marks_Hidden;
 extern int Slrn_Warn_Followup_To;
 
@@ -73,11 +72,15 @@ extern int Slrn_Use_Tildes;
 extern int Slrn_Generate_Email_From;
 extern int Slrn_Emphasized_Text_Mode;
 extern int Slrn_Emphasized_Text_Mask;
-extern int Slrn_New_Subject_Breaks_Threads;
+extern int Slrn_Highlight_Urls;
 extern int Slrn_Sig_Is_End_Of_Article;
 extern int Slrn_Del_Article_Upon_Read;
 extern int Slrn_Followup_Strip_Sig;
+extern int Slrn_Smart_Quote;
 extern int Slrn_Process_Verbatim_Marks;
+#if SLRN_HAS_UUDEVIEW
+extern int Slrn_Use_Uudeview;
+#endif
 
 #endif				       /* NOT SLRNPULL_CODE */
 
@@ -114,14 +117,14 @@ typedef struct Slrn_Header_Type
    unsigned long hash;		       /* based on msgid */
    int number;			       /* server number */
    int lines;
-   char *subject;			       /* malloced */
-   char *from;			       /* pointers to above space */
-   char *date;
+   int bytes;
+   char *subject;		       /* malloced separately */
+   char *from;			       /* malloced */
+   char *date;			       /* pointers to above space */
    char *msgid;
    char *refs;
    char *xref;
-   char *realname;		       /* NOT Null terminated */
-   unsigned int realname_len;
+   char *realname;		       /* malloced */
    unsigned int tag_number;
    Slrn_Header_Line_Type *add_hdrs;
 #define MAX_TREE_SIZE 256
@@ -138,9 +141,9 @@ extern Slrn_Header_Type *Slrn_First_Header;
 extern Slrn_Header_Type *Slrn_Current_Header;
 
 extern int slrn_goto_header (Slrn_Header_Type *, int);
+extern void slrn_set_header_flags (Slrn_Header_Type *, unsigned int);
 extern int slrn_locate_header_by_msgid (char *, int, int);
 
-
 typedef struct Slrn_Article_Line_Type
 {
    struct Slrn_Article_Line_Type *next, *prev;
@@ -161,7 +164,6 @@ typedef struct Slrn_Article_Line_Type
 #define QUOTES_LEVEL_MASK	0x1000
    /* The last few bits are devoted to quote levels. */
 #define QUOTE_LEVEL_SHIFT 12	       /* do NOT change this */
-#define MAX_QUOTE_LEVELS   8
 #define QUOTE_LEVEL(flags) ((flags) >> QUOTE_LEVEL_SHIFT)
 
    char *buf;
@@ -182,6 +184,11 @@ typedef struct
 #if SLRN_HAS_VERBATIM_MARKS
    int verbatim_hidden;
    int verbatim_marks_hidden;
+#endif
+#if SLRN_HAS_MIME
+   int mime_was_modified;
+   int mime_was_parsed;
+   int mime_needs_metamail;
 #endif
    int needs_sync;		       /* non-zero if line number/current line needs updated */
 }
@@ -205,7 +212,7 @@ extern SLRegexp_Type *Slrn_Strip_Was_Regexp[];
 extern int slrn_pipe_article_to_cmd (char *);
 extern int slrn_save_current_article (char *);
 
-extern Slrn_Article_Line_Type *slrn_search_article (char *, char **, int, int);
+extern Slrn_Article_Line_Type *slrn_search_article (char *, char **, int, int, int);
 extern unsigned int slrn_header_down_n (unsigned int, int);
 extern unsigned int slrn_header_up_n (unsigned int, int);
 
@@ -225,11 +232,11 @@ extern int slrn_get_article_window_size (void);
 extern char *slrn_extract_header (char *, unsigned int);
 
 /* Third argument must be zero unless caller deals with Slrn_Current_Header. */
-extern Slrn_Header_Type *slrn_set_header_score (Slrn_Header_Type *, int, int);
+#include "score.h"
+extern Slrn_Header_Type *slrn_set_header_score (Slrn_Header_Type *, int, int,
+						Slrn_Score_Debug_Info_Type *);
 extern void slrn_apply_scores (int);
 
-extern void slrn_sort_by_sorting_mode (void);
-
 extern int slrn_is_article_visible (void);
 extern int slrn_is_article_win_zoomed (void);
 
@@ -239,15 +246,27 @@ int slrn_get_next_pagedn_action (void);
 
 extern int slrn_string_to_article (char *str);
 
-extern int Slrn_Display_Score;
 extern int Slrn_Color_By_Score;
 extern int Slrn_Highlight_Unread;
 
 extern int slrn_set_header_format (unsigned int, char *);
 
 extern void slrn_art_sync_article (Slrn_Article_Type *);
+extern int slrn_art_get_unread (void);
+
+/* These are in art_sort.c : */
+#ifndef SLRNPULL_CODE
+extern int Slrn_New_Subject_Breaks_Threads;
+extern int Slrn_Sorting_Mode;
+extern int Slrn_Sort_By_Threads;
+extern char *Slrn_Sort_Order;
+extern int Slrn_Uncollapse_Threads;
+#endif				       /* NOT SLRNPULL_CODE */
+extern void slrn_sort_headers (void);
+
+/* These are in art_misc.c : */
 extern int _slrn_art_unhide_quotes (Slrn_Article_Type *a);
-extern int _slrn_art_hide_quotes (Slrn_Article_Type *a, int level);
+extern int _slrn_art_hide_quotes (Slrn_Article_Type *a, int);
 #if SLRN_HAS_SPOILERS
 extern void slrn_art_mark_spoilers (Slrn_Article_Type *a);
 #endif
@@ -259,7 +278,6 @@ extern void _slrn_art_skip_quoted_text (Slrn_Article_Type *a);
 extern int _slrn_art_skip_digest_forward (Slrn_Article_Type *a);
 extern char *slrn_art_extract_header (char *, unsigned int);
 extern char *slrn_cur_extract_header (char *, unsigned int);
-extern char *slrn_extract_add_header (Slrn_Header_Type *, char *);
 extern void _slrn_art_hide_headers (Slrn_Article_Type *a);
 extern void _slrn_art_unhide_headers (Slrn_Article_Type *a);
 extern int _slrn_art_unfold_header_lines (Slrn_Article_Type *a);
@@ -277,6 +295,4 @@ extern void _slrn_art_unhide_verbatim (Slrn_Article_Type *a);
 extern void slrn_art_mark_verbatim (Slrn_Article_Type *a);
 #endif
 
-/* These are in artmisc.c */
-
 #endif				       /* _SLRN_ART_H */
