@@ -14,8 +14,8 @@
  more details.
 
  You should have received a copy of the GNU General Public License along
- with this program; if not, write to the Free Software Foundation, Inc., 675
- Mass Ave, Cambridge, MA 02139, USA. 
+ with this program; if not, write to the Free Software Foundation, Inc.,
+ 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 /* This file is only included by chmap.c.  It is separated from it for easy
  * maintainence of the various character sets.
@@ -25,8 +25,9 @@
 #define HAVE_NEXT_CHARMAP	1
 #define HAVE_IBM850_CHARMAP	1
 #define HAVE_IBM852_CHARMAP	1
+#define HAVE_IBM737_CHARMAP	1
 
-#define MAX_CHARMAPS		5      /* add 1 for ISO Latin (default) */
+#define MAX_CHARMAPS		6      /* add 1 for ISO Latin (default) */
 
 typedef struct
 {
@@ -37,89 +38,85 @@ typedef struct
 }
 CharMap_Type;
 
+#define BAD 63	/* replace untranslatable chars with question marks */
+
 #if HAVE_IBM850_CHARMAP
 static CharMap_Type IBM850_Char_Map =
 {
    "ibm850", 128,
+     /* These mappings were generated mechanically by Free `recode' 3.5
+      * and edited manually -- recode preserved the graphic chars of cp850
+      * by representing them with the ISO control characters 0x80-0x9F;
+      * however, some editors were reported to choke on those. */
    /* to_iso_map */
      {
-     /* Conversion table from `cp850' charset to `iso88591' charset.
-      * Generated mechanically by GNU recode 3.4.
-      * 
-      *  The recoding should be reversible.
-      */
-	0,   1,   2,   3,   4,   5,   6,   7,	/*   0 -   7 */
-	8,   9,  10,  11,  12,  13,  14,  15,	/*   8 -  15 */
-	16,  17,  18,  19,  20,  21,  22,  23,	/*  16 -  23 */
-	24,  25,  26,  27,  28,  29,  30,  31,	/*  24 -  31 */
-	32,  33,  34,  35,  36,  37,  38,  39,	/*  32 -  39 */
-	40,  41,  42,  43,  44,  45,  46,  47,	/*  40 -  47 */
-	48,  49,  50,  51,  52,  53,  54,  55,	/*  48 -  55 */
-	56,  57,  58,  59,  60,  61,  62,  63,	/*  56 -  63 */
-	64,  65,  66,  67,  68,  69,  70,  71,	/*  64 -  71 */
-	72,  73,  74,  75,  76,  77,  78,  79,	/*  72 -  79 */
-	80,  81,  82,  83,  84,  85,  86,  87,	/*  80 -  87 */
-	88,  89,  90,  91,  92,  93,  94,  95,	/*  88 -  95 */
-	96,  97,  98,  99, 100, 101, 102, 103,	/*  96 - 103 */
-	104, 105, 106, 107, 108, 109, 110, 111,	/* 104 - 111 */
-	112, 113, 114, 115, 116, 117, 118, 119,	/* 112 - 119 */
-	120, 121, 122, 123, 124, 125, 126, 127,	/* 120 - 127 */
-	199, 252, 233, 226, 228, 224, 229, 231,	/* 128 - 135 */
-	234, 235, 232, 239, 238, 236, 196, 197,	/* 136 - 143 */
-	201, 230, 198, 244, 246, 242, 251, 249,	/* 144 - 151 */
-	255, 214, 220, 248, 163, 216, 215, 159,	/* 152 - 159 */
-	225, 237, 243, 250, 241, 209, 170, 186,	/* 160 - 167 */
-	191, 174, 172, 189, 188, 161, 171, 187,	/* 168 - 175 */
-	155, 157, 141, 129, 139, 193, 194, 192,	/* 176 - 183 */
-	169, 150, 132, 175, 184, 162, 165, 151,	/* 184 - 191 */
-	183, 181, 147, 128, 142, 143, 227, 195,	/* 192 - 199 */
-	131, 144, 146, 133, 135, 153, 158, 164,	/* 200 - 207 */
-	240, 208, 202, 203, 200, 134, 205, 206,	/* 208 - 215 */
-	207, 137, 130, 136, 154, 166, 204, 152,	/* 216 - 223 */
-	211, 223, 212, 210, 245, 213, 145, 222,	/* 224 - 231 */
-	254, 218, 219, 217, 253, 221, 140, 180,	/* 232 - 239 */
-	173, 177, 149, 190, 182, 167, 247, 148,	/* 240 - 247 */
-	176, 168, 156, 185, 179, 178, 138, 160	/* 248 - 255 */
+	0,   1,   2,   3,   4,   5,   6,   7,	/*   0 -   7  */
+	  8,   9,  10,  11,  12,  13,  14,  15,	/*   8 -  15  */
+	  16,  17,  18,  19,  20,  21,  22,  23,	/*  16 -  23  */
+	  24,  25,  26,  27,  28,  29,  30,  31,	/*  24 -  31  */
+	  32,  33,  34,  35,  36,  37,  38,  39,	/*  32 -  39  */
+	  40,  41,  42,  43,  44,  45,  46,  47,	/*  40 -  47  */
+	  48,  49,  50,  51,  52,  53,  54,  55,	/*  48 -  55  */
+	  56,  57,  58,  59,  60,  61,  62,  63,	/*  56 -  63  */
+	  64,  65,  66,  67,  68,  69,  70,  71,	/*  64 -  71  */
+	  72,  73,  74,  75,  76,  77,  78,  79,	/*  72 -  79  */
+	  80,  81,  82,  83,  84,  85,  86,  87,	/*  80 -  87  */
+	  88,  89,  90,  91,  92,  93,  94,  95,	/*  88 -  95  */
+	  96,  97,  98,  99, 100, 101, 102, 103,	/*  96 - 103  */
+	  104, 105, 106, 107, 108, 109, 110, 111,	/* 104 - 111  */
+	  112, 113, 114, 115, 116, 117, 118, 119,	/* 112 - 119  */
+	  120, 121, 122, 123, 124, 125, 126, 127,	/* 120 - 127  */
+	  199, 252, 233, 226, 228, 224, 229, 231,	/* 128 - 135  */
+	  234, 235, 232, 239, 238, 236, 196, 197,	/* 136 - 143  */
+	  201, 230, 198, 244, 246, 242, 251, 249,	/* 144 - 151  */
+	  255, 214, 220, 248, 163, 216, 215, BAD,	/* 152 - 159  */
+	  225, 237, 243, 250, 241, 209, 170, 186,	/* 160 - 167  */
+	  191, 174, 172, 189, 188, 161, 171, 187,	/* 168 - 175  */
+	  BAD, BAD, BAD, BAD, BAD, 193, 194, 192,	/* 176 - 183  */
+	  169, BAD, BAD, BAD, BAD, 162, 165, BAD,	/* 184 - 191  */
+	  BAD, BAD, BAD, BAD, BAD, BAD, 227, 195,	/* 192 - 199  */
+	  BAD, BAD, BAD, BAD, BAD, BAD, BAD, 164,	/* 200 - 207  */
+	  240, 208, 202, 203, 200, BAD, 205, 206,	/* 208 - 215  */
+	  207, BAD, BAD, BAD, BAD, 166, 204, BAD,	/* 216 - 223  */
+	  211, 223, 212, 210, 245, 213, 181, 254,	/* 224 - 231  */
+	  222, 218, 219, 217, 253, 221, 175, 180,	/* 232 - 239  */
+	  173, 177, BAD, 190, 182, 167, 247, 184,	/* 240 - 247  */
+	  176, 168, 183, 185, 179, 178, BAD, 160,	/* 248 - 255  */
      },
    /* from_iso_map */
      {
-     /* Conversion table from `iso88591' charset to `cp850' charset.
-      * Generated mechanically by GNU recode 3.4.
-      *
-      * The recoding should be reversible.
-      */
-	0,   1,   2,   3,   4,   5,   6,   7,	/*   0 -   7 */
-	8,   9,  10,  11,  12,  13,  14,  15,	/*   8 -  15 */
-	16,  17,  18,  19,  20,  21,  22,  23,	/*  16 -  23 */
-	24,  25,  26,  27,  28,  29,  30,  31,	/*  24 -  31 */
-	32,  33,  34,  35,  36,  37,  38,  39,	/*  32 -  39 */
-	40,  41,  42,  43,  44,  45,  46,  47,	/*  40 -  47 */
-	48,  49,  50,  51,  52,  53,  54,  55,	/*  48 -  55 */
-	56,  57,  58,  59,  60,  61,  62,  63,	/*  56 -  63 */
-	64,  65,  66,  67,  68,  69,  70,  71,	/*  64 -  71 */
-	72,  73,  74,  75,  76,  77,  78,  79,	/*  72 -  79 */
-	80,  81,  82,  83,  84,  85,  86,  87,	/*  80 -  87 */
-	88,  89,  90,  91,  92,  93,  94,  95,	/*  88 -  95 */
-	96,  97,  98,  99, 100, 101, 102, 103,	/*  96 - 103 */
-	104, 105, 106, 107, 108, 109, 110, 111,	/* 104 - 111 */
-	112, 113, 114, 115, 116, 117, 118, 119,	/* 112 - 119 */
-	120, 121, 122, 123, 124, 125, 126, 127,	/* 120 - 127 */
-	195, 179, 218, 200, 186, 203, 213, 204,	/* 128 - 135 */
-	219, 217, 254, 180, 238, 178, 196, 197,	/* 136 - 143 */
-	201, 230, 202, 194, 247, 242, 185, 191,	/* 144 - 151 */
-	223, 205, 220, 176, 250, 177, 206, 159,	/* 152 - 159 */
-	255, 173, 189, 156, 207, 190, 221, 245,	/* 160 - 167 */
-	249, 184, 166, 174, 170, 240, 169, 187,	/* 168 - 175 */
-	248, 241, 253, 252, 239, 193, 244, 192,	/* 176 - 183 */
-	188, 251, 167, 175, 172, 171, 243, 168,	/* 184 - 191 */
-	183, 181, 182, 199, 142, 143, 146, 128,	/* 192 - 199 */
-	212, 144, 210, 211, 222, 214, 215, 216,	/* 200 - 207 */
-	209, 165, 227, 224, 226, 229, 153, 158,	/* 208 - 215 */
-	157, 235, 233, 234, 154, 237, 231, 225,	/* 216 - 223 */
-	133, 160, 131, 198, 132, 134, 145, 135,	/* 224 - 231 */
-	138, 130, 136, 137, 141, 161, 140, 139,	/* 232 - 239 */
-	208, 164, 149, 162, 147, 228, 148, 246,	/* 240 - 247 */
-	155, 151, 163, 150, 129, 236, 232, 152	/* 248 - 255 */
+	0,   1,   2,   3,   4,   5,   6,   7,	/*   0 -   7  */
+	  8,   9,  10,  11,  12,  13,  14,  15,	/*   8 -  15  */
+	  16,  17,  18,  19,  20,  21,  22,  23,	/*  16 -  23  */
+	  24,  25,  26,  27,  28,  29,  30,  31,	/*  24 -  31  */
+	  32,  33,  34,  35,  36,  37,  38,  39,	/*  32 -  39  */
+	  40,  41,  42,  43,  44,  45,  46,  47,	/*  40 -  47  */
+	  48,  49,  50,  51,  52,  53,  54,  55,	/*  48 -  55  */
+	  56,  57,  58,  59,  60,  61,  62,  63,	/*  56 -  63  */
+	  64,  65,  66,  67,  68,  69,  70,  71,	/*  64 -  71  */
+	  72,  73,  74,  75,  76,  77,  78,  79,	/*  72 -  79  */
+	  80,  81,  82,  83,  84,  85,  86,  87,	/*  80 -  87  */
+	  88,  89,  90,  91,  92,  93,  94,  95,	/*  88 -  95  */
+	  96,  97,  98,  99, 100, 101, 102, 103,	/*  96 - 103  */
+	  104, 105, 106, 107, 108, 109, 110, 111,	/* 104 - 111  */
+	  112, 113, 114, 115, 116, 117, 118, 119,	/* 112 - 119  */
+	  120, 121, 122, 123, 124, 125, 126, 127,	/* 120 - 127  */
+	  BAD, BAD, BAD, BAD, BAD, BAD, BAD, BAD,	/* 128 - 135  */
+	  BAD, BAD, BAD, BAD, BAD, BAD, BAD, BAD,	/* 136 - 143  */
+	  BAD, BAD, BAD, BAD, BAD, BAD, BAD, BAD,	/* 144 - 151  */
+	  BAD, BAD, BAD, BAD, BAD, BAD, BAD, BAD,	/* 152 - 159  */
+	  255, 173, 189, 156, 207, 190, 221, 245,	/* 160 - 167  */
+	  249, 184, 166, 174, 170, 240, 169, 238,	/* 168 - 175  */
+	  248, 241, 253, 252, 239, 230, 244, 250,	/* 176 - 183  */
+	  247, 251, 167, 175, 172, 171, 243, 168,	/* 184 - 191  */
+	  183, 181, 182, 199, 142, 143, 146, 128,	/* 192 - 199  */
+	  212, 144, 210, 211, 222, 214, 215, 216,	/* 200 - 207  */
+	  209, 165, 227, 224, 226, 229, 153, 158,	/* 208 - 215  */
+	  157, 235, 233, 234, 154, 237, 232, 225,	/* 216 - 223  */
+	  133, 160, 131, 198, 132, 134, 145, 135,	/* 224 - 231  */
+	  138, 130, 136, 137, 141, 161, 140, 139,	/* 232 - 239  */
+	  208, 164, 149, 162, 147, 228, 148, 246,	/* 240 - 247  */
+	  155, 151, 163, 150, 129, 236, 231, 152,	/* 248 - 255  */
      }
 };
 #endif
@@ -208,6 +205,90 @@ static CharMap_Type IBM852_Char_Map =
 };
 #endif
 
+#if HAVE_IBM737_CHARMAP
+static CharMap_Type IBM737_Char_Map =
+{
+   "ibm737", 128,
+   /* to_iso_map */
+     {
+     /* Conversion table from `cp737' charset to `iso88597' charset.
+      * Generated mechanically by transtab  0.97.
+      */
+	    0,     1,     2,     3,     4,     5,     6,     7,	/* 000-007 */
+	    8,     9,    10,    11,    12,    13,    14,    15,	/* 008-015 */
+	   16,    17,    18,    19,    20,    21,    22,    23,	/* 016-023 */
+	   24,    25,    26,    27,    28,    29,    30,    31,	/* 024-031 */
+	   32,    33,    34,    35,    36,    37,    38,    39,	/* 032-039 */
+	   40,    41,    42,    43,    44,    45,    46,    47,	/* 040-047 */
+	   48,    49,    50,    51,    52,    53,    54,    55,	/* 048-055 */
+	   56,    57,    58,    59,    60,    61,    62,    63,	/* 056-063 */
+	   64,    65,    66,    67,    68,    69,    70,    71,	/* 064-071 */
+	   72,    73,    74,    75,    76,    77,    78,    79,	/* 072-079 */
+	   80,    81,    82,    83,    84,    85,    86,    87,	/* 080-087 */
+	   88,    89,    90,    91,    92,    93,    94,    95,	/* 088-095 */
+	   96,    97,    98,    99,   100,   101,   102,   103,	/* 096-103 */
+	  104,   105,   106,   107,   108,   109,   110,   111,	/* 104-111 */
+	  112,   113,   114,   115,   116,   117,   118,   119,	/* 112-119 */
+	  120,   121,   122,   123,   124,   125,   126,   127,	/* 120-127 */
+	  193,   194,   195,   196,   197,   198,   199,   200,	/* 128-135 */
+	  201,   202,   203,   204,   205,   206,   207,   208,	/* 136-143 */
+	  209,   211,   212,   213,   214,   215,   216,   217,	/* 144-151 */
+	  225,   226,   227,   228,   229,   230,   231,   232,	/* 152-159 */
+	  233,   234,   235,   236,   237,   238,   239,   240,	/* 160-167 */
+	  241,   243,   242,   244,   245,   246,   247,   248,	/* 168-175 */
+	  BAD,   BAD,   BAD,   BAD,   BAD,   BAD,   BAD,   BAD,	/* 176-183 */
+	  BAD,   BAD,   BAD,   BAD,   BAD,   BAD,   BAD,   BAD,	/* 184-191 */
+	  BAD,   BAD,   BAD,   BAD,   BAD,   BAD,   BAD,   BAD,	/* 192-199 */
+	  BAD,   BAD,   BAD,   BAD,   BAD,   BAD,   BAD,   BAD,	/* 200-207 */
+	  BAD,   BAD,   BAD,   BAD,   BAD,   BAD,   BAD,   BAD,	/* 208-215 */
+	  BAD,   BAD,   BAD,   BAD,   BAD,   BAD,   BAD,   BAD,	/* 216-223 */
+	  249,   220,   221,   222,   250,   223,   252,   253,	/* 224-231 */
+	  251,   254,   182,   184,   185,   186,   188,   190,	/* 232-239 */
+	  191,   177,   BAD,   BAD,   BAD,   BAD,   BAD,   BAD,	/* 240-247 */
+	  176,   163,   183,   BAD,   BAD,   178,   BAD,   160	/* 248-255 */
+     },
+   /* from_iso_map */
+     {
+     /* Conversion table from `iso88597' charset to `cp737' charset.
+      * Generated mechanically by transtab  0.97.
+      */
+ 	    0,     1,     2,     3,     4,     5,     6,     7,	/* 000-007 */
+	    8,     9,    10,    11,    12,    13,    14,    15,	/* 008-015 */
+	   16,    17,    18,    19,    20,    21,    22,    23,	/* 016-023 */
+	   24,    25,    26,    27,    28,    29,    30,    31,	/* 024-031 */
+	   32,    33,    34,    35,    36,    37,    38,    39,	/* 032-039 */
+	   40,    41,    42,    43,    44,    45,    46,    47,	/* 040-047 */
+	   48,    49,    50,    51,    52,    53,    54,    55,	/* 048-055 */
+	   56,    57,    58,    59,    60,    61,    62,    63,	/* 056-063 */
+	   64,    65,    66,    67,    68,    69,    70,    71,	/* 064-071 */
+	   72,    73,    74,    75,    76,    77,    78,    79,	/* 072-079 */
+	   80,    81,    82,    83,    84,    85,    86,    87,	/* 080-087 */
+	   88,    89,    90,    91,    92,    93,    94,    95,	/* 088-095 */
+	   96,    97,    98,    99,   100,   101,   102,   103,	/* 096-103 */
+	  104,   105,   106,   107,   108,   109,   110,   111,	/* 104-111 */
+	  112,   113,   114,   115,   116,   117,   118,   119,	/* 112-119 */
+	  120,   121,   122,   123,   124,   125,   126,   127,	/* 120-127 */
+	  BAD,   BAD,   BAD,   BAD,   BAD,   BAD,   BAD,   BAD,	/* 128-135 */
+	  BAD,   BAD,   BAD,   BAD,   BAD,   BAD,   BAD,   BAD,	/* 136-143 */
+	  BAD,   BAD,   BAD,   BAD,   BAD,   BAD,   BAD,   BAD,	/* 144-151 */
+	  BAD,   BAD,   BAD,   BAD,   BAD,   BAD,   BAD,   BAD,	/* 152-159 */
+	  255,   BAD,   BAD,   249,   BAD,   BAD,   BAD,   BAD,	/* 160-167 */
+	  BAD,   BAD,   BAD,   BAD,   BAD,   BAD,   BAD,   BAD,	/* 168-175 */
+	  248,   241,   253,   BAD,   BAD,   BAD,   234,   250,	/* 176-183 */
+	  235,   236,   237,   BAD,   238,   BAD,   239,   240,	/* 184-191 */
+	  BAD,   128,   129,   130,   131,   132,   133,   134,	/* 192-199 */
+	  135,   136,   137,   138,   139,   140,   141,   142,	/* 200-207 */
+	  143,   144,   BAD,   145,   146,   147,   148,   149,	/* 208-215 */
+	  150,   151,   BAD,   BAD,   225,   226,   227,   229,	/* 216-223 */
+	  BAD,   152,   153,   154,   155,   156,   157,   158,	/* 224-231 */
+	  159,   160,   161,   162,   163,   164,   165,   166,	/* 232-239 */
+	  167,   168,   170,   169,   171,   172,   173,   174,	/* 240-247 */
+	  175,   224,   228,   232,   230,   231,   233,   BAD	/* 248-255 */
+
+     }
+};
+#endif
+
 #if HAVE_NEXT_CHARMAP
 static CharMap_Type NeXT_Char_Map =
 {
@@ -382,6 +463,9 @@ static CharMap_Type *Char_Maps [MAX_CHARMAPS] =
 #if HAVE_IBM852_CHARMAP
    &IBM852_Char_Map,
 #endif
+#if HAVE_IBM737_CHARMAP
+   &IBM737_Char_Map,
+#endif
 #if HAVE_NEXT_CHARMAP
    &NeXT_Char_Map,
 #endif
