@@ -3,7 +3,7 @@
  This file is part of SLRN.
 
  Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
- Copyright (c) 2001 Thomas Schultz <tststs@gmx.de>
+ Copyright (c) 2001, 2002 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -89,6 +89,7 @@ int Slrn_No_Autosave = 0;
 int Slrn_Unsubscribe_New_Groups = 0;
 int Slrn_Check_New_Groups = 1;
 int Slrn_Drop_Bogus_Groups = 1;
+int Slrn_Max_Queued_Groups = 20;
 
 SLKeyMap_List_Type *Slrn_Group_Keymap;
 int *Slrn_Prefix_Arg_Ptr;
@@ -1029,7 +1030,7 @@ void slrn_select_prev_group (void) /*{{{*/
 
 void slrn_group_quit (void) /*{{{*/
 {
-   if (Slrn_User_Wants_Confirmation
+   if ((Slrn_User_Wants_Confirmation & SLRN_CONFIRM_QUIT)
        && (Slrn_Batch == 0)
        && (slrn_get_yesno (1, _("Do you really want to quit")) <= 0)) return;
 
@@ -1103,18 +1104,22 @@ static void set_current_group (void) /*{{{*/
 
 /*}}}*/
 
-#define MAX_QUEUED_GROUPS 20
 static void refresh_groups (Slrn_Group_Type **c) /*{{{*/
 {
    Slrn_Group_Type *g = Groups;
-   Slrn_Group_Range_Type ranges[MAX_QUEUED_GROUPS];
+   Slrn_Group_Range_Type *ranges;
+   
+   if (Slrn_Max_Queued_Groups <= 0)
+     Slrn_Max_Queued_Groups = 1;
+   ranges = (Slrn_Group_Range_Type *) slrn_safe_malloc
+     (Slrn_Max_Queued_Groups * sizeof(Slrn_Group_Range_Type));
    
    while (g != NULL)
      {
 	Slrn_Group_Type *start = g;
 	int i = 0, j = 0;
 	
-	while ((g != NULL) && (i < MAX_QUEUED_GROUPS))
+	while ((g != NULL) && (i < Slrn_Max_Queued_Groups))
 	  {
 	     if (!(g->flags & GROUP_UNSUBSCRIBED))
 	       {
@@ -1126,7 +1131,7 @@ static void refresh_groups (Slrn_Group_Type **c) /*{{{*/
 	if (Slrn_Server_Obj->sv_refresh_groups (ranges, i))
 	  {
 	     slrn_error (_("Server connection dropped."));
-	     return;
+	     goto free_and_return;
 	  }
 	g = start;
 	while (j < i)
@@ -1158,6 +1163,8 @@ static void refresh_groups (Slrn_Group_Type **c) /*{{{*/
 	while ((g != NULL) && (g->flags & GROUP_UNSUBSCRIBED))
 	  g = g->next;
      }
+   free_and_return:
+   SLfree ((char*) ranges);
 }
 /*}}}*/
 
@@ -1461,7 +1468,7 @@ static void subscribe (void) /*{{{*/
 static void catch_up (void) /*{{{*/
 {
    if ((Slrn_Group_Current_Group == NULL)
-       || (Slrn_User_Wants_Confirmation
+       || ((Slrn_User_Wants_Confirmation & SLRN_CONFIRM_CATCHUP)
 	   && (Slrn_Batch == 0)
 	   && slrn_get_yesno(1, _("Mark %s as read"), Slrn_Group_Current_Group->group_name) <= 0))
      return;
@@ -1476,7 +1483,7 @@ static void catch_up (void) /*{{{*/
 static void uncatch_up (void) /*{{{*/
 {   
    if ((Slrn_Group_Current_Group == NULL)
-       || (Slrn_User_Wants_Confirmation 
+       || ((Slrn_User_Wants_Confirmation & SLRN_CONFIRM_CATCHUP)
 	   && (Slrn_Batch == 0)
 	   && slrn_get_yesno(1, _("Mark %s as un-read"), Slrn_Group_Current_Group->group_name) <= 0))
      return;
@@ -1743,7 +1750,7 @@ void slrn_post_cmd (void) /*{{{*/
      return;
 #endif
 
-   if (Slrn_User_Wants_Confirmation && (Slrn_Batch == 0) &&
+   if ((Slrn_User_Wants_Confirmation & SLRN_CONFIRM_POST) && (Slrn_Batch == 0) &&
        (slrn_get_yesno (1, _("Are you sure that you want to post")) <= 0))
      return;
    
@@ -1839,6 +1846,10 @@ static void slrn_group_hup (int sig)
 
 static void enter_group_mode_hook (void)
 {
+   if (Slrn_Scroll_By_Page)
+     Group_Window.cannot_scroll = 2;
+   else
+     Group_Window.cannot_scroll = SLtt_Term_Cannot_Scroll;
 #if SLRN_HAS_SLANG
    slrn_run_hooks (HOOK_GROUP_MODE, 0);
 #endif
@@ -2257,16 +2268,16 @@ int slrn_read_group_descriptions (void) /*{{{*/
      {
 #ifdef VMS
 	slrn_snprintf (file, sizeof (file), "%snewsgroups.dsc",
-		       SHAREDIR);
+		       SLRN_LIB_DIR);
 	if (NULL == (fp = fopen (file, "r")))
 	  {
 	     slrn_snprintf (file, sizeof (file), "%snewsgroups-dsc",
-			    SHAREDIR);
+			    SLRN_LIB_DIR);
 	     fp = fopen (file, "r");
 	  }
 #else
 	slrn_snprintf (file, sizeof (file), "%s/newsgroups.dsc",
-		       SHAREDIR);
+		       SLRN_LIB_DIR);
 	fp = fopen (file, "r");
 #endif
 	if (fp == NULL) return -1;
