@@ -3,7 +3,7 @@
  This file is part of SLRN.
 
  Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
- Copyright (c) 2001 Thomas Schultz <tststs@gmx.de>
+ Copyright (c) 2001, 2002 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -424,6 +424,76 @@ static void set_utf8_conversion_table (void) /*{{{*/
 }
 /*}}}*/
 
+static void generic_set_regexp (SLRegexp_Type **regexp_table) /*{{{*/
+{
+#if SLANG_VERSION < 10400
+   slrn_error (_("To use this feature, please update s-lang and recompile slrn."));
+#else
+   SLang_Array_Type *at;
+   SLcmd_Cmd_Table_Type cmd_table;
+   int argc, i;
+   
+   if (-1 == SLang_pop_array_of_type (&at, SLANG_STRING_TYPE))
+     {
+	slrn_error (_("Array of string expected."));
+	return;
+     }
+   
+   if (at->num_dims != 1)
+     {
+	slrn_error (_("One-dimensional array expected."));
+	goto free_and_return;
+     }
+   
+   argc = at->dims[0];
+   
+   if ((argc<1) || (argc > SLRN_MAX_REGEXP))
+     {
+	slrn_error (_("Array must contain at least one, at most %d elements."),
+		    SLRN_MAX_REGEXP);
+	goto free_and_return;
+     }
+   
+   if (NULL == (cmd_table.string_args = (char**) slrn_malloc
+		((argc+1)*sizeof(char*), 0, 0)))
+     {
+	slrn_error (_("Failed to allocate memory."));
+	goto free_and_return;
+     }
+   
+   for (i = 0; i < argc; i++)
+     {
+	(void) SLang_get_array_element (at, &i, &(cmd_table.string_args[i+1]));
+     }
+   
+   slrn_generic_regexp_fun (argc+1, &cmd_table, regexp_table);
+   
+   slrn_free ((char*)cmd_table.string_args);
+   
+   free_and_return:
+   SLang_free_array (at);
+#endif   
+}/*}}}*/
+
+static void set_ignore_quotes (void) /*{{{*/
+{
+   generic_set_regexp (Slrn_Ignore_Quote_Regexp);
+}/*}}}*/
+
+static void set_strip_re_regexp (void) /*{{{*/
+{
+   generic_set_regexp (Slrn_Strip_Re_Regexp);
+}/*}}}*/
+
+static void set_strip_sig_regexp (void) /*{{{*/
+{
+   generic_set_regexp (Slrn_Strip_Sig_Regexp);
+}/*}}}*/
+
+static void set_strip_was_regexp (void) /*{{{*/
+{
+   generic_set_regexp (Slrn_Strip_Was_Regexp);
+}/*}}}*/
 /*}}}*/
 
 static char *get_server_name (void) /*{{{*/
@@ -688,7 +758,8 @@ static int save_current_article (char *file) /*{{{*/
 
 /*}}}*/
 
-static int generic_search_article (char *str, int is_regexp) /*{{{*/
+static int generic_search_article (char *str, int is_regexp, /*{{{*/
+				   int dir)
 {
    Slrn_Article_Line_Type *l;
    char *ptr;
@@ -696,7 +767,7 @@ static int generic_search_article (char *str, int is_regexp) /*{{{*/
    if (-1 == check_article_mode ())
      return 0;
    
-   l = slrn_search_article (str, &ptr, is_regexp, 1);
+   l = slrn_search_article (str, &ptr, is_regexp, 1, dir);
    if (l == NULL)
      return 0;
    
@@ -706,18 +777,40 @@ static int generic_search_article (char *str, int is_regexp) /*{{{*/
 
 /*}}}*/
 
+static int bsearch_article (char *s) /*{{{*/
+{
+   return generic_search_article (s, 0, 0);
+}
+/*}}}*/
+
 static int search_article (char *s) /*{{{*/
 {
-   return generic_search_article (s, 0);
+   return generic_search_article (s, 0, 1);
 }
+/*}}}*/
 
+static int search_article_first (char *s) /*{{{*/
+{
+   return generic_search_article (s, 0, 2);
+}
+/*}}}*/
+
+static int re_bsearch_article (char *s) /*{{{*/
+{
+   return generic_search_article (s, 1, 0);
+}
 /*}}}*/
 
 static int re_search_article (char *s) /*{{{*/
 {
-   return generic_search_article (s, 1);
+   return generic_search_article (s, 1, 1);
 }
+/*}}}*/
 
+static int re_search_article_first (char *s) /*{{{*/
+{
+   return generic_search_article (s, 1, 2);
+}
 /*}}}*/
 
 static char *extract_article_header (char *h) /*{{{*/
@@ -832,7 +925,7 @@ static void sort_articles (void) /*{{{*/
    if (-1 == check_article_mode ())
      return;
    
-   slrn_sort_by_sorting_mode ();
+   slrn_sort_headers ();
 }
 
 /*}}}*/
@@ -968,7 +1061,7 @@ static int get_header_tag_number (void)
 
 static void set_header_score (int *score)
 {
-   (void) slrn_set_header_score (Slrn_Current_Header, *score, 0);
+   (void) slrn_set_header_score (Slrn_Current_Header, *score, 0, NULL);
 }
 
 static int get_header_score (void)
@@ -1293,6 +1386,7 @@ static SLang_Intrin_Fun_Type Slrn_Intrinsics [] = /*{{{*/
    MAKE_INTRINSIC_0("reset_prefix_arg", reset_prefix_arg, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_SI("locate_header_by_msgid", locate_header_by_msgid, SLANG_INT_TYPE),
    MAKE_INTRINSIC_0("article_as_string", article_as_string, SLANG_STRING_TYPE),
+   MAKE_INTRINSIC_S("bsearch_article", bsearch_article, SLANG_INT_TYPE),
    MAKE_INTRINSIC_S("call", slrn_call_command, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("collapse_thread", collapse_thread, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("collapse_threads", collapse_threads, SLANG_VOID_TYPE),
@@ -1335,11 +1429,13 @@ static SLang_Intrin_Fun_Type Slrn_Intrinsics [] = /*{{{*/
    MAKE_INTRINSIC_0("popup_window", popup_window, SLANG_INT_TYPE),
    MAKE_INTRINSIC_0("prev_tagged_header", slrn_prev_tagged_header, SLANG_INT_TYPE),
    MAKE_INTRINSIC_I("quit", quit, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_S("re_bsearch_article", re_bsearch_article, SLANG_INT_TYPE),
    MAKE_INTRINSIC_S("re_bsearch_author", re_author_search_backward, SLANG_INT_TYPE),
    MAKE_INTRINSIC_S("re_bsearch_subject", re_subject_search_backward, SLANG_INT_TYPE),
    MAKE_INTRINSIC_S("re_fsearch_author", re_author_search_forward, SLANG_INT_TYPE),
    MAKE_INTRINSIC_S("re_fsearch_subject", re_subject_search_forward, SLANG_INT_TYPE),
    MAKE_INTRINSIC_S("re_search_article", re_search_article, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_S("re_search_article_first", re_search_article_first, SLANG_INT_TYPE),
    MAKE_INTRINSIC_SSS("read_mini", read_mini, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_SSS("read_mini_no_echo", read_mini_no_echo, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_SSS("read_mini_filename", read_mini_filename, SLANG_VOID_TYPE),
@@ -1349,6 +1445,7 @@ static SLang_Intrin_Fun_Type Slrn_Intrinsics [] = /*{{{*/
    MAKE_INTRINSIC_I("reload_scorefile", reload_scorefile, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_S("save_current_article", save_current_article, SLANG_INT_TYPE),
    MAKE_INTRINSIC_S("search_article", search_article, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_S("search_article_first", search_article_first, SLANG_INT_TYPE),
    MAKE_INTRINSIC_0("select_group", select_group, SLANG_INT_TYPE),
    MAKE_INTRINSIC_0("select_list_box", select_list_box, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("server_name", get_server_name, SLANG_STRING_TYPE),
@@ -1359,11 +1456,15 @@ static SLang_Intrin_Fun_Type Slrn_Intrinsics [] = /*{{{*/
    MAKE_INTRINSIC_0("hide_current_group", hide_current_group, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_I("set_header_flags", set_header_flags, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_I("set_header_score", set_header_score, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_0("set_ignore_quotes", set_ignore_quotes, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("set_input_chars", set_input_chars, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("set_input_string", set_input_string, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_SI("set_integer_variable", set_integer_variable, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_I("set_prefix_argument", set_prefix_arg, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_SS("set_string_variable", set_string_variable, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_0("set_strip_re_regexp", set_strip_re_regexp, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_0("set_strip_sig_regexp", set_strip_sig_regexp, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_0("set_strip_was_regexp", set_strip_was_regexp, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("set_utf8_conversion_table", set_utf8_conversion_table, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("get_group_order", slrn_intr_get_group_order, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("set_group_order", slrn_intr_set_group_order, SLANG_VOID_TYPE),
@@ -1425,6 +1526,9 @@ int slrn_init_slang (void) /*{{{*/
        || (-1 == SLang_init_slunix ())
 #endif
        || (-1 == SLang_init_slfile ())
+#if SLANG_VERSION >= 10400
+       || (-1 == SLang_init_import ()) /* enable dynamic linking */
+#endif
        
        /* Now add intrinsics for this application */
        || (-1 == SLadd_intrin_fun_table(Slrn_Intrinsics, NULL))
