@@ -3,6 +3,7 @@
  This file is part of SLRN.
 
  Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
+ Copyright (c) 2001, 2002 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -60,12 +61,12 @@ Menu_Type;
 
 static Menu_Type Group_Mode_Menu [] = /*{{{*/
 {
-     {"Quit", "quit"},
-     {"Refresh", "refresh_groups"},
-     {"Top", "bob"},
-     {"Bot", "eob"},
-     {"Post", "post"},
-     {"Help", "help"},
+     {N_("Quit"), "quit"},
+     {N_("Refresh"), "refresh_groups"},
+     {N_("Top"), "bob"},
+     {N_("Bot"), "eob"},
+     {N_("Post"), "post"},
+     {N_("Help"), "help"},
      {NULL, NULL}
 };
 
@@ -73,16 +74,16 @@ static Menu_Type Group_Mode_Menu [] = /*{{{*/
 	
 static Menu_Type Article_Mode_Menu [] = /*{{{*/
 {
-     {"Quit", "quit"},
-     {"Catchup", "catchup_all"},
-     {"NextGrp", "skip_to_next_group"},
-     {"NextArt", "next"},
-     {"Top", "goto_beginning"},
-     {"Bot", "goto_end"},
-     {"Post", "post"},
-     {"Reply", "reply"},
-     {"Followup", "followup"},
-     {"Help", "help"},
+     {N_("Quit"), "quit"},
+     {N_("Catchup"), "catchup_all"},
+     {N_("NextGrp"), "skip_to_next_group"},
+     {N_("NextArt"), "next"},
+     {N_("Top"), "goto_beginning"},
+     {N_("Bot"), "goto_end"},
+     {N_("Post"), "post"},
+     {N_("Reply"), "reply"},
+     {N_("Followup"), "followup"},
+     {N_("Help"), "help"},
      {NULL, NULL}
 };
 
@@ -100,7 +101,7 @@ static void update_menu (Menu_Type *m) /*{{{*/
    slrn_set_color (MENU_COLOR);
    if (m != NULL) while (m->menu_name != NULL)
      {
-	SLsmg_write_string (m->menu_name);
+	SLsmg_write_string (_(m->menu_name));
 	SLsmg_write_string ("   ");
 	m++;
      }
@@ -133,7 +134,7 @@ int slrn_execute_menu (int want_col) /*{{{*/
    col = -1;
    while (m->menu_name != NULL)
      {
-	int dcol = 2 + strlen (m->menu_name);
+	int dcol = 2 + strlen (_(m->menu_name));
 	if ((want_col > col) 
 	    && (want_col <= col + dcol))
 	  break;
@@ -151,7 +152,7 @@ int slrn_execute_menu (int want_col) /*{{{*/
      {
 	slrn_set_color (color);
 	if (col) SLsmg_write_char (' ');
-	SLsmg_write_string (m->menu_name);
+	SLsmg_write_string (_(m->menu_name));
 	SLsmg_write_char (' ');
 	SLsmg_gotorc (0, col);
 	slrn_smg_refresh ();
@@ -189,18 +190,19 @@ void slrn_update_group_menu (void) /*{{{*/
 
 static char *Sort_Selections [] = /*{{{*/
 {
-   "No sorting",		       /* 000 */
-     "Thread Headers",		       /* 001 */
-     "Sort by subject",		       /* 010 */
-     "Thread, then sort by subject.",  /* 011 */
-     "Sort by scores.",		       /* 100 */
-     "Thread, then sort by scores.",   /* 101 */
-     "Sort by score and subject",      /* 110 */
-     "Thread, then sort by score and subject",   /* 111 */
-     "Sort by date (most recent first)",		       /* 1000 */
-     "Thread, then sort by date (most recent first)",      /* 1001 */
-     "Sort by date (most recent last)",      /* 1010 */
-     "Thread, then Sort by date (most recent last)",      /* 1011 */
+   N_("No sorting"),		       /* 000 */
+     N_("Thread Headers"),		       /* 001 */
+     N_("Sort by subject"),		       /* 010 */
+     N_("Thread, then sort by subject."),  /* 011 */
+     N_("Sort by scores."),		       /* 100 */
+     N_("Thread, then sort by scores."),   /* 101 */
+     N_("Sort by score and subject"),      /* 110 */
+     N_("Thread, then sort by score and subject"),   /* 111 */
+     N_("Sort by date (most recent first)"),		       /* 1000 */
+     N_("Thread, then sort by date (most recent first)"),      /* 1001 */
+     N_("Sort by date (most recent last)"),      /* 1010 */
+     N_("Thread, then Sort by date (most recent last)"),      /* 1011 */
+     N_("Custom sorting (see manual)"),	/* 1100 */
      NULL
 };
 
@@ -208,7 +210,7 @@ static char *Sort_Selections [] = /*{{{*/
 
 static Slrn_Select_Box_Type Slrn_Sort_Select_Box = /*{{{*/
 {
-   "Sorting Method", Sort_Selections
+   N_("Sorting Method"), Sort_Selections
 };
 
 /*}}}*/
@@ -237,22 +239,25 @@ static int draw_select_box (Slrn_Select_Box_Type *sb) /*{{{*/
    char **lines, *line, *title;
    static Slrn_Select_Box_Type *last_sb;
    
-   if ((sb == NULL) && ((sb = last_sb) == NULL))
+   if (((sb == NULL) && ((sb = last_sb) == NULL)) ||
+       (sb->lines == NULL))
      return -1;
    
    last_sb = sb;
    
    lines = sb->lines;
-   title = sb->title;
+   if (sb->title == NULL)
+     title = "";
+   else
+     title = _(sb->title);
 
    slrn_push_suspension (0);
    
-   if (title == NULL) title = "";
    max_selection_len = strlen (title);
    num_selections = 0;
    while ((line = *lines) != NULL)
-     {	
-	len = strlen (line);
+     {
+	len = strlen (_(line));
 	if (len > max_selection_len) max_selection_len = len;
 	num_selections++;
 	lines++;
@@ -284,12 +289,12 @@ static int draw_select_box (Slrn_Select_Box_Type *sb) /*{{{*/
 	lines++;
 	r++;
 	SLsmg_gotorc (r, c);
-	SLsmg_printf ("%2X %s", num_selections, line);
+	SLsmg_printf ("%2X %s", num_selections, _(line));
 	num_selections++;
      }
    
    r += 2;
-   center_string_column ("(Select One)", r, column, num_columns);
+   center_string_column (_("(Select One)"), r, column, num_columns);
 				      
    slrn_set_color (FRAME_COLOR);
    SLsmg_draw_box (row, column, num_rows, num_columns);
@@ -349,7 +354,7 @@ int slrn_select_box (Slrn_Select_Box_Type *sb) /*{{{*/
    
    if ((rsp < 0) || (rsp >= num_selections))
      {
-	slrn_error ("Cancelled.");
+	slrn_error (_("Cancelled."));
 
 	while (SLang_input_pending (2))
 	  (void) SLang_getkey ();
@@ -426,11 +431,11 @@ static void draw_select_list (void)
 	
 	if (top_showing)
 	  {
-	     SLsmg_write_string (bot_showing ? "[All]" : "[Top]");
+	     SLsmg_write_string (bot_showing ? _("[All]") : _("[Top]"));
 	     Is_Popup_Window = bot_showing ? 2 : 1;
 	  }
 	else if (bot_showing)
-	  SLsmg_write_string ("[Bot]");
+	  SLsmg_write_string (_("[Bot]"));
 	else
 	  SLsmg_printf ("[%u/%u]", Select_Window->line_num, Select_Window->num_lines);
      }
@@ -470,25 +475,27 @@ static void draw_select_list (void)
    if (Is_Popup_Window)
      {
 	if (top_showing && bot_showing)
-	  slrn_message ("Press any key to close the window.");
+	  slrn_message (_("Press any key to close the window."));
 	else
-	  slrn_message ("Use UP/DOWN to move, any other key to close the window.");
+	  slrn_message (_("Use UP/DOWN to move, any other key to close the window."));
      }
    else
      {
-	slrn_message ("Use UP/DOWN to move, RETURN to select, Ctrl-G to cancel");
+	slrn_message (_("Use UP/DOWN to move, RETURN to select, Ctrl-G to cancel"));
 	SLsmg_gotorc (row + Select_Window->window_row, column);
      }
    slrn_pop_suspension ();
 }
 
-static void free_select_list (Select_List_Type *l)
+static void free_select_list (Select_List_Type *l, int free_data)
 {
    Select_List_Type *next;
    
    while (l != NULL)
      {
 	next = l->next;
+	if (free_data)
+	  slrn_free (l->data);
 	slrn_free ((char *)l);
 	l = next;
      }
@@ -591,6 +598,41 @@ static void sl_down (void)
      Select_Window->top_window_line = Select_Window->current_line;
 }
 
+static void sl_jump (void)
+{
+   char ch = (char) SLang_Last_Key_Char;
+   int dir = (ch & 0x20);
+   SLscroll_Type *startpos = Select_Window->current_line;
+   Select_List_Type *curline;
+   
+   ch |= 0x20;
+   do
+     {
+	if (dir)
+	  Select_Window->current_line = Select_Window->current_line->next;
+	else
+	  Select_Window->current_line = Select_Window->current_line->prev;
+	if (Select_Window->current_line == NULL)
+	  {
+	     Select_Window->current_line = Select_Window->lines;
+	     if (!dir)
+	       while (Select_Window->current_line->next != NULL)
+		 Select_Window->current_line = Select_Window->current_line->next;
+	  }
+	
+	if (Select_Window->current_line == startpos)
+	  {
+	     SLtt_beep ();
+	     return;
+	  }
+	curline = (Select_List_Type *) Select_Window->current_line;
+     }
+   while (ch != (*curline->data | 0x20));
+   
+   select_list_winch (0, 0);
+   SLscroll_find_line_num (Select_Window);
+}
+
 static void sl_select (void)
 {
    Select_List_Active_Color = MENU_PRESS_COLOR;
@@ -635,9 +677,13 @@ static void sl_mouse (void)
 
    
 static SLKeyMap_List_Type *Select_List_Keymap;
+static SLKeyMap_List_Type *Popup_Keymap;
 
 static int init_select_list_mode (int select_list)
 {
+   SLKeyMap_List_Type **kmap;
+   char *int_name;
+   
    slrn_free ((char *) Select_Window);
    Select_Window = (SLscroll_Window_Type *) slrn_malloc (sizeof (SLscroll_Window_Type), 1, 1);
    Is_Popup_Window = !select_list;
@@ -646,39 +692,55 @@ static int init_select_list_mode (int select_list)
    Select_Window->nrows = 5;
    select_list_winch (0, 0);
    
-   if (Select_List_Keymap != NULL)
-     return 0;
+   if (select_list)
+     {
+	kmap = &Select_List_Keymap;
+	int_name = "list";
+     }
+   else
+     {
+	kmap = &Popup_Keymap;
+	int_name = "popup";
+     }
    
-   if (NULL == (Select_List_Keymap = SLang_create_keymap ("list", NULL)))
+   if (*kmap != NULL)
+     {
+	Select_List_Mode_Cap.keymap = *kmap;
+	return 0;
+     }
+   
+   if (NULL == (*kmap = SLang_create_keymap (int_name, NULL)))
      return -1;
    
 #if defined(IBMPC_SYSTEM)
-   SLkm_define_key  ("^@H", (FVOID_STAR) sl_up, Select_List_Keymap);
-   SLkm_define_key  ("\xE0H", (FVOID_STAR) sl_up, Select_List_Keymap);
-   SLkm_define_key  ("^@P", (FVOID_STAR) sl_down, Select_List_Keymap);
-   SLkm_define_key  ("\xE0P", (FVOID_STAR) sl_down, Select_List_Keymap);
-   SLkm_define_key  ("^@I", (FVOID_STAR) sl_pageup, Select_List_Keymap);
-   SLkm_define_key  ("\xE0I", (FVOID_STAR) sl_pageup, Select_List_Keymap);
-   SLkm_define_key  ("^@Q", (FVOID_STAR) sl_pagedn, Select_List_Keymap);
-   SLkm_define_key  ("\xE0Q", (FVOID_STAR) sl_pagedn, Select_List_Keymap);
+   SLkm_define_key  ("^@H", (FVOID_STAR) sl_up, *kmap);
+   SLkm_define_key  ("\xE0H", (FVOID_STAR) sl_up, *kmap);
+   SLkm_define_key  ("^@P", (FVOID_STAR) sl_down, *kmap);
+   SLkm_define_key  ("\xE0P", (FVOID_STAR) sl_down, *kmap);
+   SLkm_define_key  ("^@I", (FVOID_STAR) sl_pageup, *kmap);
+   SLkm_define_key  ("\xE0I", (FVOID_STAR) sl_pageup, *kmap);
+   SLkm_define_key  ("^@Q", (FVOID_STAR) sl_pagedn, *kmap);
+   SLkm_define_key  ("\xE0Q", (FVOID_STAR) sl_pagedn, *kmap);
 #else
-   SLkm_define_key  ("\033[5~", (FVOID_STAR) sl_pageup, Select_List_Keymap);
-   SLkm_define_key  ("\033[6~", (FVOID_STAR) sl_pagedn, Select_List_Keymap);
-   SLkm_define_key  ("\033[A", (FVOID_STAR) sl_up, Select_List_Keymap);
-   SLkm_define_key  ("\033OA", (FVOID_STAR) sl_up, Select_List_Keymap);
-   SLkm_define_key  ("\033[B", (FVOID_STAR) sl_down, Select_List_Keymap);
-   SLkm_define_key  ("\033OB", (FVOID_STAR) sl_down, Select_List_Keymap);
+   SLkm_define_key  ("\033[5~", (FVOID_STAR) sl_pageup, *kmap);
+   SLkm_define_key  ("\033[6~", (FVOID_STAR) sl_pagedn, *kmap);
+   SLkm_define_key  ("\033[A", (FVOID_STAR) sl_up, *kmap);
+   SLkm_define_key  ("\033OA", (FVOID_STAR) sl_up, *kmap);
+   SLkm_define_key  ("\033[B", (FVOID_STAR) sl_down, *kmap);
+   SLkm_define_key  ("\033OB", (FVOID_STAR) sl_down, *kmap);
 # ifdef __unix__
-   SLkm_define_key  ("^(kd)", (FVOID_STAR) sl_down, Select_List_Keymap);
-   SLkm_define_key  ("^(ku)", (FVOID_STAR) sl_up, Select_List_Keymap);
+   SLkm_define_key  ("^(kd)", (FVOID_STAR) sl_down, *kmap);
+   SLkm_define_key  ("^(ku)", (FVOID_STAR) sl_up, *kmap);
 # endif
 #endif
-
-   SLkm_define_key  ("k", (FVOID_STAR) sl_up, Select_List_Keymap);
-   SLkm_define_key  ("j", (FVOID_STAR) sl_down, Select_List_Keymap);
+   
+   /* In select lists, these bindings collide with the new "jump" feature */
+   SLkm_define_key  ("k", (FVOID_STAR) sl_up, *kmap);
+   SLkm_define_key  ("j", (FVOID_STAR) sl_down, *kmap);
    
    if (select_list)
      {
+	char buf[2];
 #if defined(IBMPC_SYSTEM)
 	SLkm_define_key  ("^@M", (FVOID_STAR) sl_right, Select_List_Keymap);
 	SLkm_define_key  ("\xE0M", (FVOID_STAR) sl_right, Select_List_Keymap);
@@ -700,12 +762,18 @@ static int init_select_list_mode (int select_list)
 	SLkm_define_key  ("\033[M\040", (FVOID_STAR) sl_mouse, Select_List_Keymap);
 	SLkm_define_key  ("\033[M\041", (FVOID_STAR) sl_mouse, Select_List_Keymap);
 	SLkm_define_key  ("\033[M\042", (FVOID_STAR) sl_mouse, Select_List_Keymap);
+	
+	buf[1] = 0;
+	for (*buf = 'a'; *buf <= 'z'; (*buf)++)
+	  SLkm_define_key (buf, (FVOID_STAR) sl_jump, Select_List_Keymap);
+	for (*buf = 'A'; *buf <= 'Z'; (*buf)++)
+	  SLkm_define_key (buf, (FVOID_STAR) sl_jump, Select_List_Keymap);
      }
    
    if (SLang_Error)
      return -1;
    
-   Select_List_Mode_Cap.keymap = Select_List_Keymap;
+   Select_List_Mode_Cap.keymap = *kmap;
    return 0;
 }
 
@@ -738,7 +806,7 @@ int slrn_select_list_mode (char *title,
 
 	if (NULL == (curr = (Select_List_Type *) slrn_malloc (sizeof (Select_List_Type), 1, 1)))
 	  {
-	     free_select_list (root);
+	     free_select_list (root, 0);
 	     return -1;
 	  }
 
@@ -778,7 +846,7 @@ int slrn_select_list_mode (char *title,
 	slrn_do_keymap_key (Select_List_Keymap);
      }
    active_num = Select_Window->line_num - 1;
-   free_select_list (root);
+   free_select_list (root, 0);
    slrn_pop_mode ();
    
    slrn_message ("");
@@ -814,12 +882,12 @@ int slrn_popup_win_mode (char *title, char *text)
    
    do
      {
-	unsigned int len;
-	char *newline;
+	unsigned int len, pos;
+	char *newline, *tab;
 	
 	if (NULL == (curr = (Select_List_Type *) slrn_malloc (sizeof (Select_List_Type), 1, 1)))
 	  {
-	     free_select_list (root);
+	     free_select_list (root, 1);
 	     return -1;
 	  }
 	
@@ -832,7 +900,6 @@ int slrn_popup_win_mode (char *title, char *text)
 	  }
 	last = curr;
 	
-	curr->data = text;
 	if (NULL != (newline = strchr (text, '\n')))
 	  {
 	     *newline++ = 0;
@@ -840,6 +907,38 @@ int slrn_popup_win_mode (char *title, char *text)
 	       newline = NULL;
 	  }
 	len = strlen (text);
+	
+	/* Here, we handle TABs (expand to spaces): */
+	tab = text;
+	while (NULL != (tab = strchr (tab, '\t')))
+	  {
+	     tab++;
+	     len += 8;
+	  }
+	if (NULL == (curr->data = slrn_malloc (len, 1, 1)))
+	  {
+	     free_select_list (root, 1);
+	     return -1;
+	  }
+	tab = curr->data;
+	pos = 0;
+	while (*text)
+	  {
+	     if (*text == '\t')
+	       do
+	       {
+		  *tab++ = ' ';
+		  pos++;
+	       } while (pos % 8);
+	     else
+	       {
+		  *tab++ = *text;
+		  pos++;
+	       }
+	     text++;
+	  }
+	
+	len = strlen (curr->data);
 	if (len > Select_List_Window_Ncols)
 	  Select_List_Window_Ncols = len;
 	text = newline;
@@ -861,14 +960,14 @@ int slrn_popup_win_mode (char *title, char *text)
 	SLang_Key_Type *key;
 	
 	slrn_update_screen ();
-	key = SLang_do_key (Select_List_Keymap, (int (*)(void)) SLang_getkey);
+	key = SLang_do_key (Popup_Keymap, (int (*)(void)) SLang_getkey);
 	if ((key == NULL) 
 	    || (key->type == SLKEY_F_INTERPRET))
 	  Select_List_Quit = 1;
 	else
 	  key->f.f ();
      }
-   free_select_list (root);
+   free_select_list (root, 1);
    slrn_pop_mode ();
    
    slrn_message ("");
@@ -924,7 +1023,7 @@ static char *browse_dir (char *dir)
 	  return NULL;
      }
    
-   slrn_message_now ("Creating directory list...");
+   slrn_message_now (_("Creating directory list..."));
    if (NULL == (d = slrn_open_dir (dir)))
      return NULL;
    
