@@ -14,8 +14,8 @@
  more details.
 
  You should have received a copy of the GNU General Public License along
- with this program; if not, write to the Free Software Foundation, Inc., 675
- Mass Ave, Cambridge, MA 02139, USA. 
+ with this program; if not, write to the Free Software Foundation, Inc.,
+ 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 extern void slrn_update_article_menu (void);
 extern void slrn_update_group_menu (void);
@@ -34,4 +34,5 @@ extern int slrn_select_box (Slrn_Select_Box_Type *);
 
 
 extern int slrn_select_list_mode (char *, unsigned int, char **, unsigned int, int *);
+extern int slrn_popup_win_mode (char *, char *);
 extern char *slrn_browse_dir (char *);
