@@ -3,7 +3,7 @@
  This file is part of SLRN.
 
  Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
- Copyright (c) 2001 Thomas Schultz <tststs@gmx.de>
+ Copyright (c) 2001, 2002 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -16,8 +16,8 @@
  more details.
 
  You should have received a copy of the GNU General Public License along
- with this program; if not, write to the Free Software Foundation, Inc., 675
- Mass Ave, Cambridge, MA 02139, USA. 
+ with this program; if not, write to the Free Software Foundation, Inc.,
+ 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 #include "config.h"
 #include "slrnfeat.h"
@@ -68,9 +68,12 @@
 #  if defined(__WIN32__)
 #   define Win32_Winsock
 #   include <windows.h>
+#   ifdef __MINGW32__
+#    include <process.h>
+#   endif
 #  else
 #   include <netdb.h>
-#   ifndef h_errno
+#   if !defined(h_errno) && !defined(__CYGWIN__)
 extern int h_errno;
 #   endif
 #  endif 
@@ -100,13 +103,17 @@ extern int h_errno;
 #include "group.h"
 #include "slrn.h"
 #include "post.h"
-#include "server.h"
 #include "util.h"
+#include "server.h"
 #include "ttymsg.h"
 #include "art.h"
 #include "chmap.h"
 #include "snprintf.h"
 #include "mime.h"
+#include "slrndir.h"
+#include "menu.h"
+#include "hooks.h"
+#include "startup.h"
 
 #ifdef VMS
 /* valid filname chars for unix equiv of vms filename */
@@ -117,9 +124,10 @@ extern int h_errno;
 
 /*{{{ Global Variables */
 int Slrn_Full_Screen_Update = 1; 
-int Slrn_User_Wants_Confirmation = 1;
+int Slrn_User_Wants_Confirmation = SLRN_CONFIRM_ALL;
 int Slrn_Message_Present = 0;
 int Slrn_Abort_Unmodified = 0;
+int Slrn_Mail_Editor_Is_Mua = 0;
 
 #ifndef VMS
 char *Slrn_SendMail_Command;
@@ -129,17 +137,18 @@ char *Slrn_Editor;
 char *Slrn_Editor_Post;
 char *Slrn_Editor_Score;
 char *Slrn_Editor_Mail;
+int Slrn_Editor_Uses_Mime_Charset = 0;
 
 char *Slrn_Top_Status_Line;
 
 Slrn_User_Info_Type Slrn_User_Info;
 SLKeyMap_List_Type *Slrn_RLine_Keymap;
+SLang_RLine_Info_Type *Slrn_Keymap_RLI;
 
 /*}}}*/
 /*{{{ Static Variables */
 
 static int Error_Present;
-static SLang_RLine_Info_Type *Slrn_Keymap_RLI;
 static char *Input_String;
 static char *Input_String_Ptr;
 static char *Input_Chars_Ptr;
@@ -209,9 +218,9 @@ char *slrn_print_percent (char *buf, SLscroll_Window_Type *w, int lines) /*{{{*/
 		       || (w->num_lines == bot_number));
 	
 	if (w->line_num == w->window_row + 1)
-	  strcpy (buf, bot_showing ? "All" : "Top" ); /* safe */
+	  slrn_strncpy (buf, bot_showing ? _("All") : _("Top"), 66);
 	else if (bot_showing)
-	  strcpy (buf, "Bot"); /* safe */
+	  slrn_strncpy (buf, _("Bot"), 66);
 	else
 	  sprintf (buf, "%d%%", (100 * bot_number) / w->num_lines); /* safe */
     }
@@ -265,7 +274,7 @@ void slrn_update_top_status_line (void) /*{{{*/
    if (Slrn_Full_Screen_Update == 0) return;
    
    if ((fmt == NULL) || (*fmt == 0))
-     fmt = "slrn %v ** Press '?' for help, 'q' to quit. ** Server: %s";
+     fmt = _("slrn %v ** Press '?' for help, 'q' to quit. ** Server: %s");
    
    slrn_custom_printf (fmt, top_status_line_cb, NULL, 0, MENU_COLOR);
 }
@@ -332,7 +341,7 @@ static void vmessage_1 (int color, char *fmt, va_list ap)
 static void vmessage (FILE *fp, char *fmt, va_list ap)
 {
    if (Slrn_TT_Initialized & SLRN_SMG_INIT)
-     vmessage_1 (0, fmt, ap);
+     vmessage_1 (MESSAGE_COLOR, fmt, ap);
    else
      slrn_tty_vmessage (fp, fmt, ap);
 }
@@ -432,7 +441,7 @@ void slrn_error_now (unsigned secs, char *fmt, ...) /*{{{*/
      }
    slrn_smg_refresh ();
    Slrn_Message_Present = 0;
-   if (secs) sleep (secs);
+   if (secs) slrn_sleep (secs);
 }
 
 /*}}}*/
@@ -480,7 +489,7 @@ void slrn_custom_printf (char *fmt, PRINTF_CB cb, void *param, /*{{{*/
 	char *s;
 	int color = def_color;
 	int len = -1, field_len = -1;
-	int right_justify, spaces = 0;
+	int justify, spaces = 0;
 	
 	if (fmt == elsepart)
 	  {
@@ -540,11 +549,16 @@ void slrn_custom_printf (char *fmt, PRINTF_CB cb, void *param, /*{{{*/
 	
 	if (ch == '-')
 	  {
-	     right_justify = 1;
+	     justify = 1;
+	     ch = *fmt++;
+	  }
+	else if (ch == '*')
+	  {
+	     justify = 2;
 	     ch = *fmt++;
 	  }
 	else
-	  right_justify = 0;
+	  justify = 0;
 	
 	if (isdigit (ch))
 	  {
@@ -569,7 +583,7 @@ void slrn_custom_printf (char *fmt, PRINTF_CB cb, void *param, /*{{{*/
 	     
 	   case 'g':
 	     SLsmg_erase_eol ();
-	     if (right_justify)
+	     if (justify)
 	       field_len = SLtt_Screen_Cols - field_len;
 	     SLsmg_gotorc (row, field_len);
 	     break;
@@ -606,9 +620,12 @@ void slrn_custom_printf (char *fmt, PRINTF_CB cb, void *param, /*{{{*/
 	       }
 	     if (i == len + 1) spaces = 1;
 	  }
-	if (right_justify)
+	if (justify)
 	  {
-	     while (spaces)
+	     int n;
+	     if (justify == 1) n = 0; /* right justify */
+	     else n = spaces - (spaces / 2); /* center */
+	     while (spaces > n)
 	       {
 		  SLsmg_write_nchars (" ", 1);
 		  spaces--;
@@ -671,7 +688,7 @@ unsigned int slrn_toggle_format (char **formats, unsigned int cur) /*{{{*/
 int slrn_check_batch (void)
 {
    if (Slrn_Batch == 0) return 0;
-   slrn_error ("This function is not available in batch mode.");
+   slrn_error (_("This function is not available in batch mode."));
    return -1;
 }
 
@@ -727,8 +744,11 @@ void slrn_make_home_filename (char *name, char *file, size_t n) /*{{{*/
 #endif
        )
      {
-	slrn_strncpy (file, name, n);
-#if defined(IBMPC_SYSTEM)
+#ifdef __CYGWIN__
+	if (slrn_cygwin_convert_path (name, file, n))
+#endif
+	  slrn_strncpy (file, name, n);
+#if defined(IBMPC_SYSTEM) && !defined(__CYGWIN__)
 	slrn_os2_convert_path (file);
 #endif
 	return;
@@ -739,7 +759,7 @@ void slrn_make_home_filename (char *name, char *file, size_t n) /*{{{*/
    
    *file = 0;
    slrn_dircat (home, name, file, n);
-#else
+#else /* VMS */
    char *cp, *cp1;
    static char fname[SLRN_MAX_PATH_LEN];
    char fn[SLRN_MAX_PATH_LEN], fn1[SLRN_MAX_PATH_LEN];
@@ -1022,6 +1042,8 @@ FILE *slrn_open_home_file (char *name, char *mode, char *file, /*{{{*/
    (void) create_flag;
 #endif	
 
+   if (2 == slrn_file_exists (file)) /* don't open directories */
+     return NULL;
    return fopen (file, mode);
 }
 
@@ -1054,27 +1076,36 @@ int slrn_mail_file (char *file, int edit, unsigned int editline, char *to, char
    if (edit && (Slrn_Batch == 0))
      {
 	if (slrn_edit_file (Slrn_Editor_Mail, file, editline, 1) < 0) return -1;
+	if (Slrn_Mail_Editor_Is_Mua) return 0;
 	
 	while (1)
 	  {
 	     char rsp;
+   /* Note to translators: "yY" is for "yes", "nN" for "no", "eE" for "edit".
+    * Do not change the length of this string. You cannot use any of the
+    * default characters for different fields. */
+	     char *responses=_("yYnNeE");
 	     
-	     rsp = slrn_get_response ("yYnNeE", "Mail the message? \001Yes, \001No, \001Edit");
-	     rsp |= 0x20;
+	     if (strlen (responses) != 6)
+	       responses = "";
+	     rsp = slrn_get_response ("yYnNeE", responses, _("Mail the message? \001Yes, \001No, \001Edit"));
+	     rsp = slrn_map_translated_char ("yYnNeE", responses, rsp) | 0x20;
 	     if (rsp == 'n') return -1;
 	     if (rsp == 'y') break;
 	     if (slrn_edit_file (Slrn_Editor_Mail, file, 1, 0) < 0) return -1;
 	  }
      }
-   slrn_message_now ("Sending ...");
+   slrn_message_now (_("Sending ..."));
 
-   if (Slrn_Use_Mime & MIME_ARCHIVE) /* else: do it later */
+   if ((Slrn_Use_Mime & MIME_ARCHIVE) && /* else: do it later */
+       !Slrn_Editor_Uses_Mime_Charset)
      slrn_chmap_fix_file (file, 0);
    
    if (-1 == slrn_save_file_to_mail_file (file, Slrn_Save_Replies_File, NULL))
      return -1;
    
-   if ((Slrn_Use_Mime & MIME_ARCHIVE) == 0)
+   if (!(Slrn_Use_Mime & MIME_ARCHIVE) &&
+       !Slrn_Editor_Uses_Mime_Charset)
      slrn_chmap_fix_file (file, 0);
    
 #ifdef VMS
@@ -1124,6 +1155,7 @@ int slrn_mail_file (char *file, int edit, unsigned int editline, char *to, char
 		    {
 		       header = 0;
 		       
+		       slrn_add_date_header (pp);
 		       slrn_mime_add_headers (pp);
 		       fp = slrn_mime_encode (fp);
 		    }
@@ -1141,7 +1173,7 @@ int slrn_mail_file (char *file, int edit, unsigned int editline, char *to, char
 	slrn_fclose (fp);
 #  if defined(IBMPC_SYSTEM)
 	slrn_fclose (pp);
-	buf = slrn_strdup_printf ("%s %s", Slrn_SendMail_Command, outfile);
+	buf = slrn_strdup_strcat (Slrn_SendMail_Command, " ", outfile, NULL);
 	slrn_posix_system (buf, 0);
 	slrn_free (buf);
 #  else
@@ -1152,15 +1184,15 @@ int slrn_mail_file (char *file, int edit, unsigned int editline, char *to, char
 # endif /* SLRN_HAS_MIME */
      {
 # if defined(IBMPC_SYSTEM)
-	buf = slrn_strdup_printf ("%s %s", Slrn_SendMail_Command, file);
+	buf = slrn_strdup_strcat (Slrn_SendMail_Command, " ", file, NULL);
 # else
-	buf = slrn_strdup_printf ("%s < %s", Slrn_SendMail_Command, file);
+	buf = slrn_strdup_strcat (Slrn_SendMail_Command, " < ", file, NULL);
 # endif
 	slrn_posix_system (buf, 0);
 	slrn_free (buf);
      }
 #endif /* NOT VMS */
-   slrn_message ("Sending...done");
+   slrn_message (_("Sending...done"));
    return 0;
 }
 
@@ -1258,7 +1290,7 @@ static char *get_pipe_cmd (FILE *fp)
 	p = p->next;
      }
    
-   return "**UNKNOWN**";
+   return _("**UNKNOWN**");
 }
 #endif 				       /* SLRN_HAS_PIPING */
 int slrn_pclose (FILE *fp) /*{{{*/
@@ -1271,7 +1303,7 @@ int slrn_pclose (FILE *fp) /*{{{*/
    if (ret)
      {
 	char buf[SLRN_MAX_PATH_LEN];
-	fprintf (stderr, "Command %s returned exit status %d.  Press RETURN.\n", 
+	fprintf (stderr, _("Command %s returned exit status %d.  Press RETURN.\n"), 
 		 get_pipe_cmd (fp), ret);
 	fgets (buf, sizeof(buf), stdin);
      }
@@ -1298,7 +1330,7 @@ FILE *slrn_popen (char *cmd, char *mode) /*{{{*/
    if (fp == NULL)
      {
 	char buf[256];
-	fprintf (stderr, "Command %s failed to run.  Press RETURN.\n", cmd);
+	fprintf (stderr, _("Command %s failed to run.  Press RETURN.\n"), cmd);
 	fgets (buf, sizeof(buf), stdin);
 	slrn_set_display_state (SLRN_TTY_INIT | SLRN_SMG_INIT);
      }
@@ -1315,6 +1347,7 @@ FILE *slrn_popen (char *cmd, char *mode) /*{{{*/
 
 /*}}}*/
 
+/* returns a malloced string */
 static char *create_edit_command (char *edit, char *file, unsigned int line) /*{{{*/
 {
    int d, s;
@@ -1338,7 +1371,7 @@ static char *create_edit_command (char *edit, char *file, unsigned int line) /*{
 	  }
 	else
 	  {
-	     slrn_error ("Invalid Editor definition.");
+	     slrn_error (_("Invalid Editor definition."));
 	     return NULL;
 	  }
 	p++;
@@ -1351,7 +1384,7 @@ static char *create_edit_command (char *edit, char *file, unsigned int line) /*{
    p = edit;
    while ((*p != ' ') && (*p != '\t') && (*p != 0))
      {
-	if (*p == '/') *p = '\\';
+	if (*p == '/') *p = SLRN_PATH_SLASH_CHAR;
 	p++;
      }
 #endif
@@ -1360,7 +1393,7 @@ static char *create_edit_command (char *edit, char *file, unsigned int line) /*{
    
    if ((d == 0) && (s == 0))
      {
-	return slrn_strdup_printf ("%s %s", edit, file);
+	return slrn_strdup_strcat (edit, " ", file, NULL);
      }
    else if (d == 0)
      {
@@ -1370,7 +1403,7 @@ static char *create_edit_command (char *edit, char *file, unsigned int line) /*{
      {
 	char *retval, *cmd1;
 	cmd1 = slrn_strdup_printf (edit, (int) line);
-	retval = slrn_strdup_printf ("%s %s", cmd1, file);
+	retval = slrn_strdup_strcat (cmd1, " ", file, NULL);
 	slrn_free (cmd1);
 	return retval;
      }
@@ -1416,7 +1449,7 @@ int slrn_edit_file (char *editor, char *file, unsigned int line,
 #  ifdef __unix__
 	editor = "vi";
 #  else
-	slrn_error ("No editor command defined.");
+	slrn_error (_("No editor command defined."));
 	return -1;
 #  endif
 # endif
@@ -1438,14 +1471,16 @@ int slrn_edit_file (char *editor, char *file, unsigned int line,
 	ret = WEXITSTATUS(ret);
 	if (ret == 127)
 	  {
-	     msg = "The editor could not be found.";
+	     msg = _("The editor could not be found.");
 	     ret = -1;
 	  }
 	else if (ret == 126)
 	  {
-	     msg = "The editor was found, but could not be executed.";
+	     msg = _("The editor was found, but could not be executed.");
 	     ret = -1;
 	  }
+	else if (ret)
+	  msg = _("The editor returned a non-zero status.");
      }
 #endif
    
@@ -1458,20 +1493,29 @@ int slrn_edit_file (char *editor, char *file, unsigned int line,
        && (0 == stat (file, &st))
        && (mtime == (unsigned long) st.st_mtime))
      {
-	msg = "File was not modified";
+	if (msg == NULL)
+	  msg = _("File was not modified.");
 	ret = -2;
      }
 
-#ifdef IBMPC_SYSTEM
+#if defined(IBMPC_SYSTEM) && !defined(__CYGWIN__)
    if ((strlen (cmd) > 1)
        && (cmd[1] == ':')
        && (cmd[2] != '\\'))
      {
-	msg = "Please use double '\\\\' to separate directories in editor_command";
+	msg = _("Please use double '\\\\' to separate directories in editor_command");
      }
 #endif
 
-   if (msg != NULL) slrn_message (msg);
+   if (msg != NULL)
+     {
+	slrn_message (msg);
+	if (ret >= 0)
+	  {
+	     slrn_update_screen ();
+	     slrn_sleep (2);
+	  }
+     }
    slrn_free (cmd);
    slrn_free (editcmd);
    return ret;
@@ -1546,19 +1590,6 @@ static SLang_RLine_Info_Type *init_readline (void) /*{{{*/
 
 /*}}}*/
 
-int slrn_init_readline (void) /*{{{*/
-{
-   if ((Slrn_Keymap_RLI == NULL) 
-       && (NULL == (Slrn_Keymap_RLI = init_readline ())))
-     return -1;
-   
-   Slrn_RLine_Keymap = Slrn_Keymap_RLI->keymap;
-
-   return 0;
-}   
-
-/*}}}*/
-
 static int read_from_input_string (char *str)
 {
    char *s;
@@ -1611,6 +1642,358 @@ void slrn_set_input_chars (char *s)
    Input_Chars_Ptr = s;
 }
 
+/* The char * argument is expected to be SLRN_MAX_PATH_LEN in size */
+static int (*Complete_Open) (char *);
+static int (*Complete_Next) (char *);
+
+static char File_Pattern[SLRN_MAX_PATH_LEN], Dir_Name[SLRN_MAX_PATH_LEN];
+static Slrn_Dir_Type *Dir;
+static int In_Completion;
+
+/* buf needs room for at least SLRN_MAX_PATH_LEN chars */
+static int dir_findnext (char *buf) /*{{{*/
+{
+   Slrn_Dirent_Type *dirent;
+   unsigned int len = strlen (File_Pattern);
+   
+   while (NULL != (dirent = slrn_read_dir (Dir)))
+     {
+	char *name = dirent->name;
+	if ((*name == '.') && ((*(name+1) == 0) ||
+			       ((*(name+1) == '.') && (*(name+2) == 0))))
+	  continue;
+	if (!strncmp (dirent->name, File_Pattern, len))
+	  {
+	     strcpy (buf, Dir_Name); /* safe */
+	     len = strlen (buf);
+	     slrn_strncpy (buf+len, dirent->name, SLRN_MAX_PATH_LEN - len - 1);
+	     return 1;
+	  }
+     }
+   return 0;
+}
+/*}}}*/
+
+static int dir_findfirst (char *buf) /*{{{*/
+{
+   int pos = strlen (buf);
+   
+   if (Dir != NULL)
+     {
+	slrn_close_dir (Dir);
+	Dir = NULL;
+     }
+   
+   while ((pos >= 0) && (buf[pos] != SLRN_PATH_SLASH_CHAR))
+     pos--;
+   
+   if (pos == -1)
+     {
+	unsigned int len;
+	if (NULL == slrn_getcwd (Dir_Name, sizeof (Dir_Name) - 1))
+	  return 0;
+	len = strlen (Dir_Name);
+	Dir_Name[len] = SLRN_PATH_SLASH_CHAR;
+	Dir_Name[len+1] = '\0';
+     }
+   else
+     {
+	if (pos + 2 > SLRN_MAX_PATH_LEN)
+	  pos = SLRN_MAX_PATH_LEN - 2;
+	slrn_strncpy (Dir_Name, buf, pos + 2);
+     }
+   
+   pos++;
+   slrn_strncpy (File_Pattern, buf + pos, sizeof (File_Pattern));
+   
+   if (NULL == (Dir = slrn_open_dir (Dir_Name)))
+     return 0;
+
+   return dir_findnext (buf);
+}
+/*}}}*/
+
+typedef struct
+{
+   char *what;
+   void *dummy;
+}
+Generic_Var_Type;
+
+static Generic_Var_Type *Var;
+static int Var_Pos;
+
+static int var_findnext (char *buf) /*{{{*/
+{
+   unsigned int len = strlen (File_Pattern);
+   
+   while (Var->what != NULL)
+     {
+	int retval = 0;
+	if (!strncmp (Var->what, File_Pattern, len))
+	  {
+	     strcpy (buf, Dir_Name); /* safe */
+	     len = strlen (buf);
+	     slrn_strncpy (buf+len, Var->what, SLRN_MAX_PATH_LEN - len - 1);
+	     retval = 1;
+	  }
+	Var++;
+	if ((Var->what == NULL) && (Var_Pos == 0))
+	  {
+	     Var = (Generic_Var_Type*) Slrn_Str_Variables;
+	     Var_Pos++;
+	  }
+	if (retval) return 1;
+     }
+   return 0;
+}
+/*}}}*/
+
+static int var_findfirst (char *buf) /*{{{*/
+{
+   int pos = strlen (buf);
+   while ((pos > 0) && (buf[pos-1] != ' '))
+     pos--;
+   
+   slrn_strncpy (Dir_Name, buf, pos + 1);
+   strcpy (File_Pattern, buf + pos); /* safe; prompt is at most 255 chars */
+   Var = (Generic_Var_Type*) Slrn_Int_Variables; Var_Pos = 0;
+   return var_findnext (buf);
+}
+/*}}}*/
+
+static int strpcmp (char **a, char **b)
+{
+   return strcmp (*a, *b);
+}
+
+static void rli_self_insert (void) /*{{{*/
+{
+   char last_char [2];
+   last_char[0] = (char) SLang_Last_Key_Char;
+   last_char[1] = '\0';
+   SLang_rline_insert (last_char);
+   SLrline_redraw (Slrn_Keymap_RLI);
+}
+/*}}}*/
+
+static void generic_mini_complete (int cycle) /*{{{*/
+{
+   char *pl, *pb;
+   char last[SLRN_MAX_PATH_LEN], buf[SLRN_MAX_PATH_LEN];
+   static char prev[SLRN_MAX_PATH_LEN], prevcall[SLRN_MAX_PATH_LEN];
+   int n, repeat = 1; /* whether we are called with identical values again */
+   static int flag = 0, lastpoint = 0;  /* when flag goes 0, we call open */
+   char **argv = NULL;
+   unsigned int argc = 0, maxargc = 0;
+   
+   if (Complete_Open == NULL)
+     {
+	rli_self_insert ();
+	return;
+     }
+   
+   n = sizeof (buf);
+   if (Slrn_Keymap_RLI->point < n)
+     n = Slrn_Keymap_RLI->point + 1;
+   slrn_strncpy (buf, (char *) Slrn_Keymap_RLI->buf, n);
+   n = 0;
+   
+   if (strcmp (Slrn_Keymap_RLI->buf, prevcall) ||
+       (Slrn_Keymap_RLI->point != lastpoint) ||
+       (0 == In_Completion))
+     {
+	strcpy (prev, buf); /* safe */ /* save this search context */
+	flag = 0;
+	repeat = 0;
+     }
+   if (In_Completion == 2)
+     repeat = 0;
+   
+   if (cycle)
+     {
+	if (flag)
+	  flag = (*Complete_Next)(buf);
+	else if (0 == (flag = (*Complete_Open)(buf)))
+	  {
+	     rli_self_insert ();
+	     return;
+	  }
+	if (flag == 0)
+	  slrn_strncpy (buf, prev, sizeof (buf));
+	strcpy (last, buf); /* safe */
+     }
+   else
+     {
+	flag = (*Complete_Open)(buf);
+	strcpy (last, buf); /* safe */
+	
+	/* This loop tests all values from complete_next and returns the
+	 * smallest length match of initial characters of buf */
+	while (flag)
+	  {
+	     pl = last;
+	     pb = buf;
+	     
+	     if (repeat)
+	       {
+		  if (argc == maxargc)
+		    {
+		       char **newargv;
+		       maxargc += 512;
+		       newargv = (char **) slrn_realloc
+			 ((char *) argv, maxargc * sizeof (char *), 0);
+		       if (newargv == NULL)
+			 {
+			    slrn_free_argc_argv_list (argc, argv);
+			    slrn_free ((char *) argv);
+			    argc = 0;
+			 }
+		       argv = newargv;
+		    }
+		  if ((NULL != argv) &&
+		      (NULL != (argv[argc] = slrn_strmalloc (buf, 0))))
+		    argc++;
+	       }
+	     
+	     while (*pl && (*pl == *pb)) 
+	       {
+		  pl++;
+		  pb++;
+	       }
+	     
+	     *pl = 0;
+	     n++;
+	     flag = (*Complete_Next)(buf);
+	  }
+     }
+   
+   if (cycle || n)
+     {
+	unsigned int len;
+	
+	if (repeat && (argc > 1))
+	  {
+	     int sel;
+	     void (*qsort_fun) (char **, unsigned int,
+				unsigned int, int (*)(char **, char **));
+	     /* This seems to be necessary (see art_sort.c:slrn_sort_headers) */
+	     qsort_fun = (void (*)(char **, unsigned int,
+				   unsigned int, int (*)(char **, char **)))
+	       qsort;
+	     
+	     qsort_fun (argv, argc, sizeof (char *), strpcmp);
+	     
+	     sel = slrn_select_list_mode (_("Possible completions"), argc,
+					  argv, 0, NULL);
+	     slrn_update_screen ();
+	     if (sel != -1)
+	       {
+		  slrn_strncpy (last, argv[sel], sizeof (last));
+		  n = 1;
+	       }
+	  }
+	
+	len = strlen (last);
+	
+	if ((n < 2) && ((long) Complete_Open == (long) dir_findfirst) &&
+	    (flag || !cycle) && (len + 1 < sizeof (last)))
+	  {
+	     if (2 == slrn_file_exists (last))
+	       {
+		  last[len] = SLRN_PATH_SLASH_CHAR;
+		  last[len+1] = 0;
+	       }
+	  }
+	
+	slrn_strncpy ((char *) Slrn_Keymap_RLI->buf, last,
+		      Slrn_Keymap_RLI->buf_len);
+	len = strlen ((char *) Slrn_Keymap_RLI->buf);
+	Slrn_Keymap_RLI->len = Slrn_Keymap_RLI->point = len;
+	SLrline_redraw (Slrn_Keymap_RLI);
+     }
+   else SLtt_beep();
+   
+   if (!cycle)
+     strcpy (prev, last); /* safe; make this the new search context */
+   
+   slrn_free_argc_argv_list (argc, argv);
+   slrn_free ((char *) argv);
+   slrn_strncpy (prevcall, (char *) Slrn_Keymap_RLI->buf, sizeof (prevcall));
+   lastpoint = Slrn_Keymap_RLI->point;
+   In_Completion = cycle ? 2 : 1;
+   
+   return;
+}
+
+/*}}}*/
+
+static int mini_complete (void)
+{
+   generic_mini_complete (0);
+   return 0;
+}
+
+static int mini_cycle (void)
+{
+   generic_mini_complete (1);
+   return 0;
+}
+
+static int rli_del_bol (void) /*{{{*/
+{
+   unsigned char *d, *s;
+   
+   d = Slrn_Keymap_RLI->buf;
+   s = Slrn_Keymap_RLI->buf + Slrn_Keymap_RLI->point;
+   
+   while (0 != (*d++ = *s++));
+   Slrn_Keymap_RLI->len -= Slrn_Keymap_RLI->point;
+   Slrn_Keymap_RLI->point = 0;
+   SLrline_redraw (Slrn_Keymap_RLI);
+   return 0;
+}
+/*}}}*/
+
+static int rli_del_bow (void) /*{{{*/
+{
+   unsigned char *d, *s;
+   
+   if (Slrn_Keymap_RLI->point == 0) return 0;
+   
+   d = Slrn_Keymap_RLI->buf + Slrn_Keymap_RLI->point - 1;
+   while ((d != Slrn_Keymap_RLI->buf) && ((*d == ' ') || (*d == '\t')))
+     d--;
+   while (d != Slrn_Keymap_RLI->buf)
+     {
+	if ((*d == ' ') || (*d == '\t'))
+	  {
+	     d++;
+	     break;
+	  }
+	else d--;
+     }
+   s = Slrn_Keymap_RLI->buf + Slrn_Keymap_RLI->point;
+   
+   Slrn_Keymap_RLI->len -= (s - d);
+   Slrn_Keymap_RLI->point -= (s - d);
+   while (0 != (*d++ = *s++));
+   SLrline_redraw (Slrn_Keymap_RLI);
+   return 0;
+}
+/*}}}*/
+
+#define A_KEY(s, f)  {s, (int (*)(void)) f}
+
+SLKeymap_Function_Type Slrn_Custom_Readline_Functions [] =
+{
+   A_KEY("complete", mini_complete),
+   A_KEY("cycle", mini_cycle),
+   A_KEY("delbol", rli_del_bol),
+   A_KEY("delbow", rli_del_bow),
+   A_KEY(NULL, NULL)
+};
+
 /* str needs to have enough space for 256 characters */
 static int generic_read_input (char *prompt, char *dfl, char *str, int trim_flag, 
 			       int no_echo, int point) /*{{{*/
@@ -1625,28 +2008,15 @@ static int generic_read_input (char *prompt, char *dfl, char *str, int trim_flag
    
    slrn_strncpy (prompt_buf, prompt, sizeof (prompt_buf));
    len = strlen (prompt);
-   if (len + 3 < sizeof (prompt_buf))
-     {
-	if (len && (prompt[len - 1] != '?'))
-	  {
-	     strcpy (prompt_buf + len, ": "); /* safe */
-	     len += 2;
-	  }
-	else 
-	  {
-	     strcpy (prompt_buf + len, " "); /* safe */
-	     len++;
-	  }
-     }
    
    if ((dfl != NULL) && *dfl)
      {
-	if (len + 13 + strlen (dfl) < sizeof (prompt_buf))
+	if (slrn_snprintf (prompt_buf + len, sizeof (prompt_buf) - len,
+	   _("(default: %s) "), dfl) == (int) (sizeof (prompt_buf) - len))
 	  {
-	     sprintf (prompt_buf + len, /* safe */
-		      "(default: %s) ", dfl); 
+	     *(prompt_buf + len) = '\0';
+	     dfl = NULL;
 	  }
-	else dfl = NULL;
      }
    prompt = prompt_buf;
 
@@ -1703,7 +2073,9 @@ static int generic_read_input (char *prompt, char *dfl, char *str, int trim_flag
    SLang_Error = 0;
 
    Reading_Input = 1;
+   slrn_set_color (MESSAGE_COLOR);
    i = SLang_read_line (Slrn_Keymap_RLI);
+   slrn_set_color (0);
    Reading_Input = 0;
    
    slrn_enable_mouse (1);
@@ -1763,6 +2135,30 @@ int slrn_read_input_no_echo (char *prompt, char *dfl, char *str, int trim_flag,
    return generic_read_input (prompt, dfl, str, trim_flag, 1, point);
 }
 
+int slrn_read_filename (char *prompt, char *dfl, char *str, int trim_flag, int point)
+{
+   int retval;
+   Complete_Open = dir_findfirst;
+   Complete_Next = dir_findnext;
+   retval = generic_read_input (prompt, dfl, str, trim_flag, 0, point);
+   Complete_Open = NULL;
+   Complete_Next = NULL;
+   In_Completion = 0;
+   return retval;
+}
+
+int slrn_read_variable (char *prompt, char *dfl, char *str, int trim_flag, int point)
+{
+   int retval;
+   Complete_Open = var_findfirst;
+   Complete_Next = var_findnext;
+   retval = generic_read_input (prompt, dfl, str, trim_flag, 0, point);
+   Complete_Open = NULL;
+   Complete_Next = NULL;
+   In_Completion = 0;
+   return retval;
+}
+
 int slrn_read_integer (char *prompt, int *dflt, int *np) /*{{{*/
 {
    char sdfl_buf[32];
@@ -1779,13 +2175,13 @@ int slrn_read_integer (char *prompt, int *dflt, int *np) /*{{{*/
    *str = 0;
    if (-1 == (n = slrn_read_input (prompt, sdfl, str, 1, 0)))
      {
-	slrn_error ("Abort!");
+	slrn_error (_("Abort!"));
 	return -1;
      }
    
    if (1 != sscanf(str, "%d", &n))
      {
-	slrn_error ("Integer expected.");
+	slrn_error (_("Integer expected."));
 	return -1;
      }
    *np = n;
@@ -1794,7 +2190,37 @@ int slrn_read_integer (char *prompt, int *dflt, int *np) /*{{{*/
 
 /*}}}*/
 
-char slrn_get_response (char *valid_chars, char *str, ...) /*{{{*/
+int slrn_init_readline (void) /*{{{*/
+{
+   if ((Slrn_Keymap_RLI == NULL) 
+       && (NULL == (Slrn_Keymap_RLI = init_readline ())))
+     return -1;
+   
+   Slrn_RLine_Keymap = Slrn_Keymap_RLI->keymap;
+   SLkm_define_key ("\t", (FVOID_STAR) mini_complete, Slrn_RLine_Keymap);   
+   SLkm_define_key (" ", (FVOID_STAR) mini_cycle, Slrn_RLine_Keymap);
+   SLkm_define_key ("^U", (FVOID_STAR) rli_del_bol, Slrn_RLine_Keymap);
+   SLkm_define_key ("^W", (FVOID_STAR) rli_del_bow, Slrn_RLine_Keymap);
+
+   return 0;
+}
+
+/*}}}*/
+
+char slrn_map_translated_char (char *native_chars, /*{{{*/
+			       char *translated_chars, char rsp)
+{
+   char *pos;
+   if ((strlen (native_chars) != strlen (translated_chars)) ||
+       (NULL != slrn_strchr (native_chars, rsp)) ||
+       (NULL == (pos = slrn_strchr (translated_chars, rsp))))
+     return rsp;
+   return native_chars[pos - translated_chars];
+}
+/*}}}*/
+
+char slrn_get_response (char *valid_chars, char *translated_chars, /*{{{*/
+			char *str, ...)
 {
    char ch;
    va_list ap;
@@ -1830,7 +2256,7 @@ char slrn_get_response (char *valid_chars, char *str, ...) /*{{{*/
 		  slrn_clear_message ();
 		  
 		  va_start(ap, str);
-		  vmessage_1 (0, str, ap);
+		  vmessage_1 (MESSAGE_COLOR, str, ap);
 		  va_end(ap);
 		  
 		  slrn_smg_refresh ();
@@ -1845,13 +2271,19 @@ char slrn_get_response (char *valid_chars, char *str, ...) /*{{{*/
 	  }
 
 	v = valid_chars;
+	while (*v)
+	  {
+	     if (*v == ch) return ch;
+	     v++;
+	  }
+	v = translated_chars;
 	while (*v)
 	  {
 	     if (*v == ch) return ch;
 	     v++;
 	  }
 	
-	slrn_error_now (0, "Invalid response! Try again.");
+	slrn_error_now (0, _("Invalid response! Try again."));
 	if (Slrn_TT_Initialized & SLRN_TTY_INIT)
 	  {
 	     (void) SLang_input_pending (15);
@@ -1860,36 +2292,65 @@ char slrn_get_response (char *valid_chars, char *str, ...) /*{{{*/
 }
 
 /*}}}*/
+
+/*{{{ Doubles percent characters. p0 is source, p is destination. */
+static void escape_percent_chars (char *p, char *p0, char *pmax)
+{
+   while ((*p0) && (p < pmax - 2))
+     {
+	if (*p0 == '%')
+	  *p++ = '%';
+	*p++ = *p0++;
+     }
+   *p = '\0';
+}
+/*}}}*/
+
 int slrn_get_yesno (int dflt, char *str, ...) /*{{{*/
+/* For both dflt and the return value, 0 means "no", 1 is "yes". */
 {
    va_list ap;
-   char buf[512];
+   char buf0[512], buf[512];
    char ch, rsp;
-   char *fmt;
+   char *prompt, *fmt;
+   /* Note to translators:
+    * The translated string needs to have exactly four characters.
+    * The first two become valid keys for "yes", the last two for "no".
+    * You *cannot* use "y" for "no" or "n" for "yes".
+    */
+   char *responses=_("yYnN");
    
    /* if (SLang_Error) return -1; */
    
    va_start(ap, str);
-   (void) slrn_vsnprintf(buf, sizeof (buf) - 15, str, ap);
+   (void) slrn_vsnprintf(buf0, sizeof (buf0), str, ap);
    va_end(ap);
    
+   /* As the prompt will be processed through printf again (by
+    * slrn_get_response), we need to escape percent characters. */
+   escape_percent_chars (buf, buf0, buf + sizeof (buf));
+   
    if (dflt)
      {
 	ch = 'y';
-	fmt = "? [\001Y]es, \001No";
+	fmt = _("? [\001Y]es, \001No");
      }
    else
      {
 	ch = 'n';
-	fmt = "? \001Yes, [\001N]o";
+	fmt = _("? \001Yes, [\001N]o");
      }
    
-   strcat (buf, fmt); /* safe */
-   rsp = slrn_get_response ("yYnN\r", buf);
-   if (rsp == '\r') rsp = ch;
-   else rsp |= 0x20;
+   prompt = slrn_strdup_strcat (buf, fmt, NULL);
+   if (strlen (responses) != 4) /* Translator messed it up */
+     responses = "";
+   rsp = slrn_get_response ("yYnN\r\n", responses, prompt);
+   slrn_free (prompt);
+   if ((rsp == '\r') || (rsp == '\n')) rsp = ch;
+   else rsp = slrn_map_translated_char ("yYnN", responses, rsp) | 0x20;
    
-   if (rsp == 'n') return 0;
+   if (rsp == 'n')
+     return 0;
    return 1;
 }
 
@@ -1898,29 +2359,33 @@ int slrn_get_yesno_cancel (char *str, ...) /*{{{*/
 {
    va_list ap;
    char buf[512];
+   char rsp, *prompt;
+   /* Note to translators:
+    * The translated string needs to have exactly six characters.
+    * The first two become valid keys for "yes", the next two for "no",
+    * the last ones for "cancel". You cannot use the default characters
+    * for other fields than they originally stood for.
+    */
+   char *responses=_("yYnNcC");
+   if (strlen (responses) != 6)
+     responses = "";
    
    if (SLang_Error) return -1;
    
    va_start(ap, str);
-   (void) slrn_vsnprintf(buf, sizeof(buf) - 25, str, ap);
+   (void) slrn_vsnprintf(buf, sizeof(buf), str, ap);
    va_end(ap);
    
-   strcat (buf, "? [\001Y]es, \001No, \001Cancel"); /* safe */
+   prompt = slrn_strdup_strcat (buf, _("? [\001Y]es, \001No, \001Cancel"), NULL);
+   rsp = slrn_get_response ("\007yYnNcC\r", responses, prompt);
+   slrn_free (prompt);
+   if (rsp == '\r') rsp = 'y';
+   else if (rsp == 7) rsp = 'c';
+   else rsp = slrn_map_translated_char ("yYnNcC", responses, rsp) | 0x20;
    
-   switch (slrn_get_response ("\007yYnNcC\r", buf))
-     {
-      case '\r':
-      case 'Y':
-      case 'y':
-	return 1;
-	
-      case 'n':
-      case 'N':
-	return 0;
-	
-      default:
-	return -1;
-     }
+   if (rsp == 'y') return 1;
+   if (rsp == 'n') return 0;
+   return -1;
 }
 
 /*}}}*/
@@ -1942,7 +2407,7 @@ void slrn_evaluate_cmd (void) /*{{{*/
    char buf[256];
    
    *buf = '\0';
-   if (slrn_read_input ("S-Lang>", NULL, buf, 0, 0) > 0)
+   if (slrn_read_input ("S-Lang> ", NULL, buf, 0, 0) > 0)
      {
 	SLang_load_string (buf);
      }
@@ -1967,7 +2432,7 @@ SLRegexp_Type *slrn_compile_regexp_pattern (char *pat) /*{{{*/
    
    if (0 != SLang_regexp_compile (&re))
      {
-	slrn_error ("Invalid regular expression or expression too long.");
+	slrn_error (_("Invalid regular expression or expression too long."));
 	return NULL;
      }
    return &re;
@@ -2035,7 +2500,7 @@ static char *get_hostname (void)
 #if defined(TRY_AGAIN) && !defined(MULTINET)
    if ((host_entry == NULL) && (h_errno == TRY_AGAIN))
      {
-	sleep (2);
+	slrn_sleep (2);
 	host_entry = gethostbyname (buf);
      }
 #endif
@@ -2095,13 +2560,13 @@ void slrn_get_user_info (void) /*{{{*/
 #endif
    
    /* Fill in what is assumed to be non-NULL by rest of program. */
-   Slrn_User_Info.followup_string = slrn_safe_strmalloc ("In article %m, %r wrote:");
-   Slrn_User_Info.reply_string = slrn_safe_strmalloc ("In %n, you wrote:");
+   Slrn_User_Info.followup_string = slrn_safe_strmalloc (_("In article %m, %r wrote:"));
+   Slrn_User_Info.reply_string = slrn_safe_strmalloc (_("In %n, you wrote:"));
 
    Slrn_CC_Followup_Message =
-     slrn_safe_strmalloc ("[This message has also been posted to %n.]");
+     slrn_safe_strmalloc (_("[This message has also been posted to %n.]"));
    Slrn_CC_Post_Message =
-     slrn_safe_strmalloc ("[This message has also been posted.]");
+     slrn_safe_strmalloc (_("[This message has also been posted.]"));
    
    /* Now get default values for rest. */
    host = get_hostname ();
@@ -2231,43 +2696,132 @@ void slrn_get_user_info (void) /*{{{*/
 
 /*}}}*/
 
+#define is_atext(x) (((x) > 0x20) && ((x) < 0x7f) && \
+			(NULL == strchr ("\"(),.:;<>@[\\]", (x))))
+
+static int is_dot_atom (char *str)
+{
+   char ch = *str++;
+   if (!is_atext (ch)) return 0;
+   while ((ch = *str++))
+     {
+	if (ch == '.') ch = *str++;
+	if (!is_atext (ch)) return 0;
+     }
+   return 1;
+}
+
+static int is_phrase (char *str)
+{
+   char ch;
+   while ((ch = *str++))
+     if ((!is_atext (ch)) && (0 == (ch & 0x80)) &&
+	 (ch != ' ') && (ch != '\t')) return 0;
+   /* Note: 8bit characters are encoded later and become atext */
+   return 1;
+}
+
+static int make_quoted_string (char *src, char *dest, size_t n)
+{
+   size_t len = 1;
+   char ch;
+   
+   if (n == 0) return -1;
+   
+   *dest++ = '"';
+   while (((ch = *src++)) && (len + 1 < n))
+     {
+	if ((ch == 0x0a) || (ch == 0x0d)) continue;
+	if ((ch == '\t') || (ch == '"') || (ch == '\\'))
+	  {
+	     *dest++ = '\\';
+	     len++;
+	  }
+	*dest++ = ch;
+	len++;
+     }
+   
+   if (len + 2 > n) return -1;
+   *dest++ = '"'; *dest = '\0';
+   return 0;
+}
+
+/* returns -1 upon failure */
+static int make_localpart (char *username, char *dest, size_t n)
+{
+   if ((username == NULL) || (*username == 0) || (strlen (username) >= n))
+     return -1;
+   if (is_dot_atom (username))
+     {
+	strcpy (dest, username); /* safe */
+	return 0;
+     }
+   return make_quoted_string (username, dest, n);
+}
+
+static int make_realname (char *realname, char *dest, size_t n)
+{
+   if ((realname == NULL) || (*realname == 0) || (strlen (realname) >= n))
+     return -1;
+   if (is_phrase (realname))
+     {
+	strcpy (dest, realname); /* safe */
+	return 0;
+     }
+   return make_quoted_string (realname, dest, n);
+}
+
 char *slrn_make_from_string (void)
 {
-   char *username, *hostname, *realname;
    static char buf[256];
+   char localpart[128], realname[128], *msg;
+   unsigned int addrlen = 0;
 
 #if SLRN_HAS_SLANG && ! SLRN_HAS_STRICT_FROM
-   if ((1 == SLang_run_hooks ("make_from_string_hook", 0))
-       && (0 == SLang_pop_slstring (&username)))
+   if ((1 == slrn_run_hooks (HOOK_MAKE_FROM_STRING, 0))
+       && (0 == SLang_pop_slstring (&msg)))
      {
-	if (*username != 0)
+	if (*msg != 0)
 	  {
-	     slrn_strncpy (buf, username, sizeof (buf));
-	     SLang_free_slstring (username);
+	     slrn_strncpy (buf, msg, sizeof (buf));
+	     SLang_free_slstring (msg);
 	     return buf;
 	  }
-	SLang_free_slstring (username);
+	SLang_free_slstring (msg);
 	/* Drop through to default */
      }
+   msg = NULL;
 #endif
 
-   if (NULL == (username = Slrn_User_Info.username))
-     username = "";
-
-   if (NULL == (hostname = Slrn_User_Info.hostname))
-     hostname = "";
-
-   if (NULL == (realname = Slrn_User_Info.realname))
-     realname = "";
-
-   /* Build string of the form "username@hostname (realname)" */
-   slrn_snprintf (buf, sizeof (buf), "%s@%s", username, hostname);
-   if (*realname != 0)
+   if (make_localpart (Slrn_User_Info.username, localpart, sizeof (localpart)))
+     msg = _("Cannot generate \"From:\" line without a valid username.");
+   else if ((NULL == Slrn_User_Info.hostname) ||
+	    (0 == *Slrn_User_Info.hostname))
+     /* Note: we currently do not check whether hostname is valid */
+     msg = _("Cannot generate \"From:\" line without a hostname.");
+   else
      {
-	char *p = buf + strlen (buf);
-	if (strlen (realname) + 3 < sizeof(buf) - (size_t) (p - buf))
-	  sprintf (p, " (%s)", realname); /* safe */
+	addrlen = strlen (localpart) + strlen (Slrn_User_Info.hostname) + 2;
+	if (addrlen > sizeof (buf))
+	  msg = _("This client cannot handle addresses longer than 255 characters.");
+     }
+   
+   if (msg != NULL)
+     {
+	slrn_error (msg);
+	return NULL;
+     }
+   
+   if ((0 == make_realname (Slrn_User_Info.realname, realname,
+			    sizeof (realname))) &&
+       (addrlen + strlen (realname) + 3 <= sizeof (buf)))
+     {
+	sprintf (buf, "%s <%s@%s>", realname, localpart, /* safe */
+		 Slrn_User_Info.hostname);
+     }
+   else
+     {
+	sprintf (buf, "%s@%s", localpart, Slrn_User_Info.hostname); /* safe */
      }
    return buf;
 }
-
