@@ -15,8 +15,8 @@
  more details.
 
  You should have received a copy of the GNU General Public License along
- with this program; if not, write to the Free Software Foundation, Inc., 675
- Mass Ave, Cambridge, MA 02139, USA. 
+ with this program; if not, write to the Free Software Foundation, Inc.,
+ 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 #include "config.h"
 #include "slrnfeat.h"
@@ -43,6 +43,10 @@
 # include "vms.h"
 #endif
 
+extern int Slrn_Prefer_Head;
+
+int Slrn_Force_Authentication = 0;
+
 static Slrn_Server_Obj_Type NNTP_Server_Obj;
 static Slrn_Post_Obj_Type NNTP_Post_Obj;
 
@@ -60,7 +64,7 @@ static void _nntp_connection_lost_hook (NNTP_Type *s)
 	if ((_NNTP_Abort_On_Disconnection == -1) && SLKeyBoard_Quit)
 	  return;
 
-	slrn_exit_error ("Server connection to %s lost.  Cannot recover.", 
+	slrn_exit_error (_("Server connection to %s lost.  Cannot recover."), 
 			 s->host);
      }
 }
@@ -104,7 +108,9 @@ static int _nntp_initialize_server (void)
    if (NULL == (NNTP_Server = nntp_open_server (NNTP_Server_Name, NNTP_Port)))
      return -1;
    
-   if (-1 == nntp_has_cmd (NNTP_Server, "XOVER"))
+   if (Slrn_Prefer_Head == 2)
+     NNTP_Server->can_xover = 0;
+   else if (-1 == nntp_has_cmd (NNTP_Server, "XOVER"))
      {
 	_nntp_close_server ();
 	return -1;
@@ -112,12 +118,13 @@ static int _nntp_initialize_server (void)
    
    if (NNTP_Server->can_xover == 0)
      {
-	slrn_message_now ("Server %s does not implement the XOVER command.", 
+	slrn_message_now (_("Server %s does not implement the XOVER command."), 
 			  NNTP_Server->host);
 	NNTP_Server_Obj.sv_has_xover = 0;
      }
    else NNTP_Server_Obj.sv_has_xover = 1;
    
+   NNTP_Post_Obj.po_can_post = NNTP_Server->can_post;
    NNTP_Server->flags |= NNTP_RECONNECT_OK;
 
    NNTP_Connection_Lost_Hook = _nntp_connection_lost_hook;
@@ -160,6 +167,11 @@ static int _nntp_select_group (char *grp, int *min, int *max)
    return nntp_select_group (NNTP_Server, grp, min, max);
 }
 
+static int _nntp_refresh_groups (Slrn_Group_Range_Type *gr, int n)
+{
+   return nntp_refresh_groups (NNTP_Server, gr, n);
+}
+
 static char *_nntp_current_group (void)
 {
    return NNTP_Server->group_name;
@@ -210,7 +222,7 @@ static int _nntp_list_active (char *pat)
    code = nntp_list_active_cmd (NNTP_Server, pat);
    
    if (OK_GROUPS == code)
-     _NNTP_Abort_On_Disconnection = 1;
+     _NNTP_Abort_On_Disconnection = (pat == NULL) ? 1 : -1;
 
    return code;
 }
@@ -244,13 +256,13 @@ static int _nntp_end_post (void)
    
    if (status == -1)
      {
-	slrn_error ("Error writing to server.");
+	slrn_error (_("Error writing to server."));
 	return -1;
      }
    
    if (NNTP_Server->code != OK_POSTED)
      {
-	slrn_error ("Article rejected: %s", NNTP_Server->rspbuf);
+	slrn_error (_("Article rejected: %s"), NNTP_Server->rspbuf);
 	return -1;
      }
 
@@ -340,6 +352,7 @@ static int nntp_init_objects (void)
    NNTP_Post_Obj.po_can_post = 1;
    
    NNTP_Server_Obj.sv_select_group = _nntp_select_group;
+   NNTP_Server_Obj.sv_refresh_groups = _nntp_refresh_groups;
    NNTP_Server_Obj.sv_current_group = _nntp_current_group;
    NNTP_Server_Obj.sv_read_line = _nntp_read_line;
    NNTP_Server_Obj.sv_close = _nntp_close_server;
@@ -409,11 +422,11 @@ static int nntp_select_post_object (void)
 
 static void nntp_usage (void)
 {
-   fputs ("--nntp options:\n\
+   fputs (_("--nntp options:\n\
 -h nntp-host    Host name to connect to.  This overrides NNTPSERVER variable.\n\
 -p NNTP-PORT    Set the NNTP port to NNTP-PORT. The default value is 119.\n\
                  Note: This option has no effect on some systems.\n\
-",
+"),
 	  stdout);
    exit (0);
 }
