@@ -2,7 +2,7 @@
  This file is part of SLRN.
 
  Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
- Copyright (c) 2001 Thomas Schultz <tststs@gmx.de>
+ Copyright (c) 2001, 2002 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -784,8 +784,8 @@ static int post_references_header (char *line)
 {
 #define GNKSA_LENGTH 986 /* 998 - strlen ("References: ") */
    char buf[GNKSA_LENGTH + 2];
-   char *p, *l, *r, *nextid;
-   unsigned int len = 0;
+   char *p, *l, *r, *nextid, *tmp;
+   unsigned int len = 0, extra_whitespaces=0;
 
    /* Make sure line does not end in whitespace */
    (void) slrn_trim_string (line);
@@ -800,11 +800,23 @@ static int post_references_header (char *line)
    len = r - l + 1;
    if (nextid != NULL) /* Skip enough IDs to fit into our limit */
      {
-	nextid--;
+	tmp=nextid;
+	while (NULL != (tmp = slrn_strchr (tmp+1, '>')))
+	/* make sure that we have enough space for missing whitespaces
+	 * between Message-Ids */
+	  {
+	     if (*(tmp+1) != ' ') extra_whitespaces++;
+	  }
+        nextid--;
 	while (NULL != (nextid = slrn_strchr (nextid + 1, '<')))
 	  {
-	     if (strlen (nextid) + len < GNKSA_LENGTH)
+	     if (strlen (nextid) + extra_whitespaces + len < GNKSA_LENGTH)
 	       break;
+	     else
+	       {
+		  tmp = slrn_strchr (nextid, '>');
+		  if (*(tmp+1) != ' ') extra_whitespaces--;
+	       }
 	  }
      }
    
@@ -842,7 +854,7 @@ static int post_references_header (char *line)
    return 0;
 }
 
-static int saved_failed_post (char *file)
+static int saved_failed_post (char *file, char *msg)
 {
    FILE *fp, *outfp;
    char filebuf[SLRN_MAX_PATH_LEN];
@@ -865,8 +877,9 @@ static int saved_failed_post (char *file)
 	return -1;
      }
 
-   slrn_message (_("Failed post saved in %s"), filebuf);
-
+   slrn_error (_("%s Failed post saved in %s"),
+	       (msg != NULL) ? msg : _("Posting not allowed."), filebuf);
+   
    (void) fputs ("\n\n", outfp);
 
    while (NULL != (fgets (line, sizeof (line), fp)))
@@ -1086,10 +1099,9 @@ int slrn_post_file (char *file, char *to, int is_postponed)
 	(void) slrn_run_hooks (HOOK_POST_FILE, 1, file);
 	if (SLang_Error)
 	  {
-	     slrn_error (_("post_file_hook returned error.  %s not posted."), file);
 	     if (!Slrn_Editor_Uses_Mime_Charset)
 	       slrn_chmap_fix_file (file, 1);
-	     (void) saved_failed_post (file);
+	     (void) saved_failed_post (file, _("post_file_hook returned error."));
 	     return -1;
 	  }
 
@@ -1109,9 +1121,9 @@ int slrn_post_file (char *file, char *to, int is_postponed)
 	fclose (fp);
 	if (!Slrn_Editor_Uses_Mime_Charset)
 	  slrn_chmap_fix_file (file, 1);
-	(void) saved_failed_post (file);
-	if (status != -1)
-	  slrn_error (_("Posting not allowed."));
+	(void) saved_failed_post (file, (status != -1) ?
+				  _("Posting not allowed.") :
+				  _("Could not reach server."));
 	return -1;
      }
 
@@ -1209,7 +1221,10 @@ int slrn_post_file (char *file, char *to, int is_postponed)
 	     linep[len - 1] = 0;
 #if SLRN_HAS_MIME
 	     if (Slrn_Use_Mime & MIME_DISPLAY)
-	       slrn_mime_header_encode (linep, sizeof (line) - 1);
+	       {
+		  slrn_mime_header_encode (linep, sizeof (line) - 1);
+		  len = strlen (linep);
+	       }
 #endif
 	  }
 
@@ -1244,7 +1259,7 @@ int slrn_post_file (char *file, char *to, int is_postponed)
 	  slrn_chmap_fix_file (file, 1); 
 	if (Slrn_Batch) 
 	  {
-	     saved_failed_post (file);
+	     saved_failed_post (file, NULL);
 	     return -1;
 	  }
 
@@ -1267,7 +1282,7 @@ int slrn_post_file (char *file, char *to, int is_postponed)
 	    || ((rsp == 'e')
 		&& (slrn_edit_file (Slrn_Editor_Post, file, 1, 0) < 0)))
 	  {
-	     (void) saved_failed_post (file);
+	     (void) saved_failed_post (file, NULL);
 	     return -1;
 	  }
 	goto try_again;
