@@ -15,10 +15,11 @@
  more details.
 
  You should have received a copy of the GNU General Public License along
- with this program; if not, write to the Free Software Foundation, Inc., 675
- Mass Ave, Cambridge, MA 02139, USA. 
+ with this program; if not, write to the Free Software Foundation, Inc.,
+ 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 extern void slrn_add_signature (FILE *);
+extern void slrn_add_date_header (FILE *);
 extern int slrn_add_custom_headers (FILE *, char *, int (*)(char *, FILE *));
 extern int slrn_post (char *, char *, char *);
 extern int slrn_post_file (char *, char *, int);
@@ -33,5 +34,6 @@ extern char *Slrn_Post_Custom_Headers;
 extern int Slrn_Reject_Long_Lines;
 extern char *Slrn_Postpone_Dir;
 extern int Slrn_Generate_Message_Id;
+extern int Slrn_Generate_Date_Header;
 extern char *Slrn_Signoff_String;
 extern int Slrn_Netiquette_Warnings;
