@@ -2,7 +2,7 @@
  This file is part of SLRN.
 
  Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
- Copyright (c) 2001 Thomas Schultz <tststs@gmx.de>
+ Copyright (c) 2001, 2002 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -78,8 +78,9 @@ typedef struct PScore_Regexp_Type
 #define SCORE_MESSAGE_ID 7
 #define SCORE_DATE 8
 #define SCORE_AGE 9
-#define SCORE_SUB_AND 10
-#define SCORE_SUB_OR 11
+#define SCORE_BYTES 10
+#define SCORE_SUB_AND 11
+#define SCORE_SUB_OR 12
    /* generic requires extra server interaction */
 #define SCORE_GENERIC 16
    unsigned int flags;
@@ -104,6 +105,11 @@ typedef struct PScore_Type
 #define SCORE_IS_OR_TYPE 2
    struct PScore_Regexp_Type *pregexp_list;
    struct PScore_Type *next;
+
+   /* the following variables are used by the score-debugger */
+   const char *filename;
+   unsigned int linenumber;
+   const char *description;
 }
 PScore_Type;
 
@@ -162,6 +168,37 @@ typedef struct Group_Score_Type
 }
 Group_Score_Type;
 
+typedef struct Scorefile_Name_Type
+{
+   const char *filename;
+   struct Scorefile_Name_Type *next;
+}
+Scorefile_Name_Type;
+
+Scorefile_Name_Type *Scorefile_Names = NULL;
+
+static Slrn_Score_Debug_Info_Type *append_score_debug_info
+  (Slrn_Score_Debug_Info_Type *prev, PScore_Type *sc)
+{
+   Slrn_Score_Debug_Info_Type *newentry;
+   
+   if ((newentry = (Slrn_Score_Debug_Info_Type *)slrn_malloc
+	(sizeof (Slrn_Score_Debug_Info_Type), 0, 0)) == NULL)
+     return NULL;
+   
+   newentry->filename = sc->filename;
+   newentry->description = sc->description;
+   newentry->linenumber = sc->linenumber;
+   newentry->score = sc->score;
+   newentry->stop_here = sc->flags & RETURN_THIS_SCORE;
+   newentry->next = NULL;
+   
+   if (prev != NULL)
+     prev->next = newentry;
+   
+   return newentry;
+}
+
 static Group_Score_Type *Group_Score_Root;
 static Group_Score_Type *Group_Score_Next;
 static Score_Type *Score_Root, *Score_Tail;
@@ -182,7 +219,11 @@ static int match_srt (Slrn_Header_Type *h, Score_Regexp_Type *srt,
           {
            case SCORE_LINES:
 	     ival = h->lines;
-	     goto integer_compare;/* ugly, ugly, ugly!! */
+	     goto integer_compare;
+	     
+	   case SCORE_BYTES:
+	     ival = h->bytes;
+	     goto integer_compare;
 		  
            case SCORE_SUBJECT:
              s = h->subject;
@@ -356,8 +397,10 @@ static int match_srt (Slrn_Header_Type *h, Score_Regexp_Type *srt,
    else return 1;
 }
 
-int slrn_score_header (Slrn_Header_Type *h, char *newsgroup)
+int slrn_score_header (Slrn_Header_Type *h, char *newsgroup,
+		       Slrn_Score_Debug_Info_Type **info)
 {
+   Slrn_Score_Debug_Info_Type *prev = NULL;
    Score_Type *st;
    int score = 0;
    int or_type;
@@ -384,6 +427,14 @@ int slrn_score_header (Slrn_Header_Type *h, char *newsgroup)
           {
 	     int st_score = st->pscore->score;
 	     
+	     if (info != NULL)
+	       {
+		  if (*info == NULL)
+		    *info = prev = append_score_debug_info (NULL, st->pscore);
+		  else if (prev != NULL)
+		    prev = append_score_debug_info (prev, st->pscore);
+	       }
+	     
 	     if (st->pscore->flags & RETURN_THIS_SCORE)
 	       return st_score;
 #if 0
@@ -479,27 +530,26 @@ static int chain_group_regexp (PScore_Type *pst, int *generic)
 	if (st == NULL)
 	  return -1;
 	
-	if (Score_Root == NULL)
-	  {
-	     Score_Root = st;
-	  }
-	else Score_Tail->next = st;
-	
-	Score_Tail = st;
-	
 	st->pscore = pst;
-	
 	psrt = pst->pregexp_list;
 	srt = &st->regexp_list;
 	
 	if (compile_psrt(psrt, srt, generic) != 0)
-	  return -1;
+	  slrn_free ((char*)st);
+	else
+	  {
+	     if (Score_Root == NULL) Score_Root = st;
+	     else Score_Tail->next = st;
+	     Score_Tail = st;
+	  }
 	
 	pst = pst->next;
      } /* while (pst != NULL) */
    return 0;
 }
 
+char *Slrn_Scorefile_Open = NULL;
+
 static void free_group_chain (void)
 {
    Score_Regexp_Type *srt;
@@ -523,12 +573,21 @@ static void free_group_chain (void)
 
 int slrn_open_score (char *group_name)
 {
-   Group_Score_Type *gsc = Group_Score_Root;
+   Group_Score_Type *gsc;
    unsigned int n;
    int generic;
 
    free_group_chain ();
    
+   if (Slrn_Scorefile_Open != NULL)
+     {
+	slrn_message_now (_("Reloading score file ..."));
+	if (-1 == slrn_read_score_file (Slrn_Scorefile_Open))
+	  slrn_error (_("Error processing score file %s."), Slrn_Scorefile_Open);
+	slrn_free (Slrn_Scorefile_Open);
+	Slrn_Scorefile_Open = NULL;
+     }
+   
    if ((Slrn_Perform_Scoring == 0) || (Group_Score_Root == NULL))
      {
 	Slrn_Apply_Score = 0;
@@ -537,6 +596,7 @@ int slrn_open_score (char *group_name)
    
    n = strlen (group_name);
    generic = 0;
+   gsc = Group_Score_Root;
    while (gsc != NULL)
      {
 	Group_Score_Name_Type *gsnt;
@@ -607,7 +667,7 @@ static int add_group_regexp (PScore_Regexp_Type *psrt, unsigned char *str,
 	len = (unsigned int) (slrn_trim_string ((char *) str) - (char *) str);
 	if (0 == len) return -1;
 	
-	if (type == SCORE_LINES)
+	if ((type == SCORE_LINES) || (type == SCORE_BYTES))
 	  {
 	     psrt->ireg.ival = atoi((char *)str);
 	     psrt->flags |= USE_INTEGER;
@@ -701,8 +761,10 @@ static int compile_group_names (char *group, Group_Score_Type *gst)
    return -1;
 }
 
-static PScore_Type *create_new_score (unsigned char *group, int new_group_flag, unsigned int gst_not_flag,
-				      int score, unsigned int pscore_flags)
+static PScore_Type *create_new_score
+  (unsigned char *group, int new_group_flag, unsigned int gst_not_flag,
+   int score, unsigned int pscore_flags,
+   const char *filename, unsigned int linenumber, const char *description)
 {
    PScore_Type *pst;
    
@@ -745,7 +807,15 @@ static PScore_Type *create_new_score (unsigned char *group, int new_group_flag,
 
    pst->score = score;
    pst->flags = pscore_flags;
-   
+
+   pst->filename = filename;
+   pst->linenumber = linenumber;
+
+   if (description != NULL)
+     pst->description = slrn_safe_strmalloc ((char *)description);
+   else
+     pst->description = slrn_safe_strmalloc ((char *)"");
+
    if (Group_Score_Next->pst == NULL)
      {
 	Group_Score_Next->pst = pst;
@@ -762,7 +832,7 @@ static PScore_Type *create_new_score (unsigned char *group, int new_group_flag,
 
 static void score_error (char *msg, char *line, unsigned int linenum, char *file)
 {
-   slrn_error ("Error processing %s\nLine %u:\n%s\n%s\n",
+   slrn_error (_("Error processing %s\nLine %u:\n%s\n%s\n"),
 	       file, linenum, line, msg);
 }
 
@@ -798,10 +868,19 @@ static void free_group_scores (void)
 		  SLFREE (r);
 		  r = rnext;
 	       }
+	     slrn_free ((char *) pst->description);
 	     SLFREE (pst);
 	     pst = pnext;
 	  }
 	
+	while (Scorefile_Names != NULL)
+	  {
+	     Scorefile_Name_Type *next = Scorefile_Names->next;
+	     slrn_free ((char *) Scorefile_Names->filename);
+	     slrn_free ((char *) Scorefile_Names);
+	     Scorefile_Names = next;
+	  }
+	
 	SLFREE (Group_Score_Root);
 	Group_Score_Root = gnext;
      }
@@ -909,6 +988,8 @@ static int phrase_score_file (char *file, FILE *fp, Score_Context_Type *c,
 {
     char line[1024];
     PScore_Regexp_Type *psrt;
+
+    char *description = NULL;
     
     while (fgets (line, sizeof (line) - 1, fp))
      {
@@ -933,8 +1014,9 @@ static int phrase_score_file (char *file, FILE *fp, Score_Context_Type *c,
 		  if (0 == handle_include_line (file, (char *)lp + 7, c))
 		    continue;
 		  
-		  score_error ("Error handling INCLUDE line", line, *linenum,
+		  score_error (_("Error handling INCLUDE line"), line, *linenum,
 			       file);
+		  slrn_free (description);
 		  return -1;
 	       }
 	     
@@ -960,7 +1042,8 @@ static int phrase_score_file (char *file, FILE *fp, Score_Context_Type *c,
 		  
 		  if ((ch != ']') || (g == gmax))
 		    {
-		       score_error ("Syntax Error.", line, *linenum, file);
+		       score_error (_("Syntax Error."), line, *linenum, file);
+		       slrn_free (description);
 		       return -1;
 		    }
 		  *g = 0;
@@ -972,7 +1055,7 @@ static int phrase_score_file (char *file, FILE *fp, Score_Context_Type *c,
         
 	     if (!slrn_case_strncmp (lp, (unsigned char *)"Score:", 6))
 	       {
-		  unsigned char *lpp = lp + 6;
+		  unsigned char *lpp = lp + 6, *eol;
 		  c->pscore_flags = 0;
 		  if (*lpp == ':')
 		    {
@@ -987,6 +1070,25 @@ static int phrase_score_file (char *file, FILE *fp, Score_Context_Type *c,
 		    }
 		  c->score = atoi ((char *)lpp);
 		  c->start_new_score = 1;
+
+                  if (description != NULL)
+		    {
+		       slrn_free (description);
+		       description = NULL;
+		    }
+                  eol = (unsigned char *) slrn_strchr ((char *)lpp, '#');
+		  if (eol == NULL)
+		    lpp = (unsigned char *) slrn_strchr ((char *)lpp, '%');
+		  else lpp = eol;
+                  if (lpp != NULL)
+		    {
+		       lpp = (unsigned char *) slrn_skip_whitespace ((char *)lpp + 1);
+		       eol = lpp + strlen ((char*)lpp) - 1;
+		       while ((eol >= lpp) && (*eol <= 32))
+			 *eol-- = '\0';
+		       description = slrn_safe_strmalloc ((char *)lpp);
+		    }
+
 		  continue;
 	       }
 	     
@@ -997,13 +1099,14 @@ static int phrase_score_file (char *file, FILE *fp, Score_Context_Type *c,
 		       ret = has_score_expired (lp + 8, c->today);
 		       if (ret == -1)
 			 {
-			    score_error ("Expecting 'Expires: MM/DD/YYYY' or 'Expires: DD-MM-YYYY'",
+			    score_error (_("Expecting 'Expires: MM/DD/YYYY' or 'Expires: DD-MM-YYYY'"),
 					 line, *linenum, file);
+			    slrn_free (description);
 			    return -1;
 			 }
 		       if (ret)
 			 {
-			    slrn_message ("%s has expired score on line %d",
+			    slrn_message (_("%s has expired score on line %d"),
 					  file, *linenum);
 			    c->start_new_score = 0;
 			    c->score_has_expired = 1;
@@ -1013,27 +1116,37 @@ static int phrase_score_file (char *file, FILE *fp, Score_Context_Type *c,
 		    }
 		  
 		  if (NULL == (c->pst = create_new_score (c->group, c->start_new_group, c->gnt_not_flag,
-							  c->score, c->pscore_flags)))
+							  c->score, c->pscore_flags,
+                                                          file, *linenum, description)))
 		    {
-		       score_error ("Bad group regular expression.", line,
+		       score_error (_("Bad group regular expression."), line,
 				    *linenum, file);
+		       slrn_free (description);
 		       return -1;
 		    }
 		  c->start_new_group = 0;
 		  c->start_new_score = 0;
 		  c->score_has_expired = 0;
+
+                  if (description != NULL)
+		    {
+		       slrn_free (description);
+		       description = NULL;
+		    }
 	       }
 	     
 	     if (c->score_has_expired) continue;
 	     
 	     if (c->pst == NULL)
 	       {
-		  score_error ("Expecting Score keyword.", line, *linenum, file);
+		  score_error (_("Expecting Score keyword."), line, *linenum, file);
 		  return -1;
 	       }
 	     
 	  } /*if (sub_psrt==null) */
 	
+	slrn_free (description);
+	
 	if (*lp == '~')
 	  {
 	     not_flag = 1;
@@ -1060,6 +1173,8 @@ static int phrase_score_file (char *file, FILE *fp, Score_Context_Type *c,
 	  ret = add_group_regexp (psrt, lp + 4, lp, SCORE_DATE, not_flag);
 	else if (!slrn_case_strncmp (lp, (unsigned char *)"Age:", 4))
 	  ret = add_group_regexp (psrt, lp + 3, lp, SCORE_AGE, not_flag);
+	else if (!slrn_case_strncmp (lp, (unsigned char *)"Bytes:", 6))
+	  ret = add_group_regexp (psrt, lp + 5, lp, SCORE_BYTES, not_flag);
 	else if (!slrn_case_strncmp (lp, (unsigned char *)"Message-Id:", 11))
 	  ret = add_group_regexp (psrt, lp + 10, lp, SCORE_MESSAGE_ID, not_flag);
         else if (!slrn_case_strncmp (lp, (unsigned char *)"{:", 1))
@@ -1080,7 +1195,7 @@ static int phrase_score_file (char *file, FILE *fp, Score_Context_Type *c,
 	       return 0;
 	     else
 	       {
-		  score_error ("Missing COLON.", line, *linenum, file);
+		  score_error (_("Missing COLON."), line, *linenum, file);
 		  return -1;
 	       }
 	  }
@@ -1090,7 +1205,7 @@ static int phrase_score_file (char *file, FILE *fp, Score_Context_Type *c,
 	     while (*lpp && (*lpp != ':')) lpp++;
 	     if (*lpp != ':')
 	       {
-		  score_error ("Missing COLON.", line, *linenum, file);
+		  score_error (_("Missing COLON."), line, *linenum, file);
 		  return -1;
 	       }
 	     
@@ -1099,7 +1214,7 @@ static int phrase_score_file (char *file, FILE *fp, Score_Context_Type *c,
 	
 	if (ret == -1)
 	  {
-	     score_error ("No regular expression given.", line, *linenum, file);
+	     score_error (_("No regular expression given."), line, *linenum, file);
 	     return -1;
 	  }
 	
@@ -1130,10 +1245,11 @@ static int phrase_score_file (char *file, FILE *fp, Score_Context_Type *c,
 	       }
 	  }
      } /*while (fgets (line, sizeof (line) - 1, fp))*/
+
    if (sub_psrt == NULL) return 0;
    else
      {
-	score_error ("Missing '}' for  subscore", line, *linenum, file);
+	score_error (_("Missing '}' for  subscore"), line, *linenum, file);
 	return -1;
      }
 }
@@ -1141,20 +1257,37 @@ static int phrase_score_file (char *file, FILE *fp, Score_Context_Type *c,
 static int read_score_file_internal (char *file, Score_Context_Type *c)
 {
    FILE *fp;
+   Scorefile_Name_Type *files = Scorefile_Names, *thisfile;
    unsigned int linenum = 0;
    int ret;
+   
+   while (files != NULL)
+     {
+	if (!strcmp (files->filename, file))
+	  {
+	     slrn_error (_("*** Warning * Read file %s before - skipping ..."), file);
+	     return 0;
+	  }
+	files = files->next;
+     }
+   
+   thisfile = (Scorefile_Name_Type *)
+     slrn_safe_malloc (sizeof (Scorefile_Name_Type));
+   thisfile->filename = slrn_safe_strmalloc (file);
+   thisfile->next = Scorefile_Names;
+   Scorefile_Names = thisfile;
 
    if (NULL == (fp = fopen (file, "r")))
      {
-	slrn_error ("Unable to open score file %s", file);
+	slrn_error (_("Unable to open score file %s"), file);
 	return -1;
      }
 
-   ret = phrase_score_file(file, fp, c, &linenum, NULL);
+   ret = phrase_score_file((char*)thisfile->filename, fp, c, &linenum, NULL);
    fclose (fp);
    return ret;
 }
-			
+
 int slrn_read_score_file (char *file)
 {
    Score_Context_Type sc;
@@ -1164,6 +1297,7 @@ int slrn_read_score_file (char *file)
      {
 	free_group_scores ();
      }
+   free_group_chain ();
 
    sc.today = get_today ();
    sc.score = 0;
@@ -1187,4 +1321,3 @@ int slrn_read_score_file (char *file)
 
    return status;
 }
-
