@@ -3,7 +3,7 @@
  This file is part of SLRN.
 
  Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
- Copyright (c) 2001 Thomas Schultz <tststs@gmx.de>
+ Copyright (c) 2001, 2002 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -1132,6 +1132,7 @@ static int fetch_head (NNTP_Type *s, int n, char **headers, Slrn_XOver_Type *xov
    int status;
    Slrn_Header_Type h;
    int score;
+   Slrn_Score_Debug_Info_Type *sdi = NULL;
    
    *headers = NULL;
 
@@ -1158,6 +1159,8 @@ static int fetch_head (NNTP_Type *s, int n, char **headers, Slrn_XOver_Type *xov
    slrn_map_xover_to_header (xov, &h);
 
 #if SLRN_HAS_MIME
+   h.subject = slrn_safe_strmalloc (xov->subject_malloced);
+   h.from = slrn_safe_strmalloc (xov->from_malloced);
    if (Slrn_Use_Mime)
      {
 	slrn_rfc1522_decode_string (h.subject);
@@ -1169,17 +1172,45 @@ static int fetch_head (NNTP_Type *s, int n, char **headers, Slrn_XOver_Type *xov
    (void) is_msgid_cached (h.msgid, Current_Newsgroup, (unsigned int) n, 1);
 #endif
 
-   score = slrn_score_header (&h, Current_Newsgroup, NULL);
+   score = slrn_score_header (&h, Current_Newsgroup, (KLog_Fp != NULL) ? &sdi : NULL);
    if (score < Kill_Score)
      {
 	Num_Killed++;
 	if (KLog_Fp != NULL)
-	  fprintf (KLog_Fp, "Score %d killed article %s\nNewsgroup: %s\nFrom: %s\nSubject: %s\n\n",
-		   score, h.msgid, Current_Newsgroup, h.from, h.subject);
+	  {
+	     Slrn_Score_Debug_Info_Type *hlp = sdi;
+	     fprintf (KLog_Fp, _("Score %d killed article %s\n"), score, h.msgid);
+	     while (hlp != NULL)
+	       {
+		  if (hlp->description [0] != 0)
+		    fprintf (KLog_Fp, _(" Score %c%5i: %s (%s:%i)\n"),
+			     (hlp->stop_here ? '=' : ' '), hlp->score,
+			     hlp->description, hlp->filename, hlp->linenumber);
+		  else
+		    fprintf (KLog_Fp, _(" Score %c%5i: %s:%i\n"),
+			     (hlp->stop_here ? '=' : ' '), hlp->score,
+			     hlp->filename, hlp->linenumber);
+		  hlp = hlp->next;
+	       }
+	     fprintf (KLog_Fp, _("  Newsgroup: %s\n  From: %s\n  Subject: %s\n\n"),
+		      Current_Newsgroup, h.from, h.subject);
+	  }
 	slrn_free (*headers);
 	*headers = NULL;
-	return 0;
+/*	return 0; */
      }
+   while (sdi != NULL)
+     {
+	Slrn_Score_Debug_Info_Type *hlp = sdi->next;
+	slrn_free ((char *)sdi);
+	sdi = hlp;
+     }
+#if SLRN_HAS_MIME
+   slrn_free (h.subject);
+   h.subject = xov->subject_malloced;
+   slrn_free (h.from);
+   h.from = xov->from_malloced;
+#endif
 #if 0
    /* This next call should add the message id to the cache. */
    (void) is_msgid_cached (h.msgid, Current_Newsgroup, (unsigned int) n, 1);
