@@ -3,6 +3,7 @@
  This file is part of SLRN.
 
  Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
+ Copyright (c) 2001, 2002 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -75,7 +76,7 @@
 # include <arpa/inet.h>
 #endif
 
-#ifndef h_errno
+#if !defined(h_errno) && !defined(__CYGWIN__)
 extern int h_errno;
 #endif
 
@@ -730,14 +731,20 @@ int sltcp_close (SLTCP_Type *tcp) /*{{{*/
 /*}}}*/
 
 #if !defined(VMS) && !defined(USE_WINSOCK_SLTCP)
-static int wait_for_input (int fd) /*{{{*/
+static int wait_for_input (SLTCP_Type *tcp) /*{{{*/
 {
    fd_set fds;
    struct timeval tv;
+   int fd = tcp->tcp_fd;
    
    if (fd == -1)
      return -1;
    
+#if SLTCP_HAS_SSL_SUPPORT
+   if (tcp->ssl != NULL && SSL_pending(tcp->ssl) > 0)
+     return 0;
+#endif
+   
    while (1)
      {
 	int ret;
@@ -814,7 +821,7 @@ static int do_read (SLTCP_Type *tcp) /*{{{*/
     * slrn attempts to set signals to NOT restart system calls.  In such a 
     * case, the read below can be interrupted.
     */
-   if (-1 == wait_for_input (fd))
+   if (-1 == wait_for_input (tcp))
      return -1;
 #endif
 
