@@ -3,7 +3,7 @@
  This file is part of SLRN.
 
  Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
- Copyright (c) 2001 Thomas Schultz <tststs@gmx.de>
+ Copyright (c) 2001, 2002 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -106,8 +106,6 @@ static SLcmd_Cmd_Type Slrn_Startup_File_Cmds[] = /*{{{*/
      {mono_fun, "mono", "SSsss"},
      {set_variable_fun, "set", "SG"},
      {nnrp_fun, "nnrpaccess", "SSS" },
-#define SLRN_MAX_REGEXP 5
-   /* used for ignore_quotes and strip_*_regexp */
      {ignore_quote_fun, "ignore_quotes", "Sssss"},
      {strip_re_fun, "strip_re_regexp", "Sssss"},
      {strip_sig_fun, "strip_sig_regexp", "Sssss"},
@@ -365,8 +363,8 @@ static SLRegexp_Type *compile_quote_regexp (char *str) /*{{{*/
 }
 
 /*}}}*/
-static void generic_regexp_fun (int argc, SLcmd_Cmd_Table_Type *cmd_table,
-				SLRegexp_Type **regexp_table) /*{{{*/
+void slrn_generic_regexp_fun (int argc, SLcmd_Cmd_Table_Type *cmd_table,
+				     SLRegexp_Type **regexp_table) /*{{{*/
 {
    unsigned int i;
    SLRegexp_Type *r;
@@ -398,25 +396,25 @@ static void generic_regexp_fun (int argc, SLcmd_Cmd_Table_Type *cmd_table,
 
 static int ignore_quote_fun (int argc, SLcmd_Cmd_Table_Type *table)
 {
-   generic_regexp_fun (argc, table, Slrn_Ignore_Quote_Regexp);
+   slrn_generic_regexp_fun (argc, table, Slrn_Ignore_Quote_Regexp);
    return 0;
 }
 
 static int strip_re_fun (int argc, SLcmd_Cmd_Table_Type *table)
 {
-   generic_regexp_fun (argc, table, Slrn_Strip_Re_Regexp);
+   slrn_generic_regexp_fun (argc, table, Slrn_Strip_Re_Regexp);
    return 0;
 }
 
 static int strip_sig_fun (int argc, SLcmd_Cmd_Table_Type *table)
 {
-   generic_regexp_fun (argc, table, Slrn_Strip_Sig_Regexp);
+   slrn_generic_regexp_fun (argc, table, Slrn_Strip_Sig_Regexp);
    return 0;
 }
 
 static int strip_was_fun (int argc, SLcmd_Cmd_Table_Type *table)
 {
-   generic_regexp_fun (argc, table, Slrn_Strip_Was_Regexp);
+   slrn_generic_regexp_fun (argc, table, Slrn_Strip_Was_Regexp);
    return 0;
 }
 
@@ -469,6 +467,7 @@ Slrn_Int_Var_Type Slrn_Int_Variables [] = /*{{{*/
      {"hide_verbatim_marks",	&Slrn_Verbatim_Marks_Hidden},
      {"hide_signature",	&Slrn_Signature_Hidden},
      {"hide_pgpsignature",	&Slrn_Pgp_Signature_Hidden},
+     {"hide_quotes",            &Slrn_Quotes_Hidden_Mode},
      {"emphasized_text_mask",	&Slrn_Emphasized_Text_Mask},
      {"emphasized_text_mode",	&Slrn_Emphasized_Text_Mode},
      {"process_verbatim_marks", &Slrn_Process_Verbatim_Marks},
@@ -521,7 +520,8 @@ Slrn_Int_Var_Type Slrn_Int_Variables [] = /*{{{*/
      {"cc_followup", &Slrn_Auto_CC_To_Poster},
      {"use_tmpdir", &Slrn_Use_Tmpdir},
      {"sorting_method", &Slrn_Sorting_Mode},
-     {"uncollapse_threads", &Slrn_Threads_Visible},
+     {"custom_sort_by_threads", &Slrn_Sort_By_Threads},
+     {"uncollapse_threads", &Slrn_Uncollapse_Threads},
      {"read_active", &Slrn_List_Active_File},
      {"drop_bogus_groups", &Slrn_Drop_Bogus_Groups},
      {"prefer_head", &Slrn_Prefer_Head},
@@ -546,6 +546,7 @@ Slrn_Int_Var_Type Slrn_Int_Variables [] = /*{{{*/
      {"wrap_method", &Slrn_Wrap_Method},
      {"write_newsrc_flags", &Slrn_Write_Newsrc_Flags},
      {"query_read_group_cutoff", &Slrn_Query_Group_Cutoff},
+     {"max_queued_groups", &Slrn_Max_Queued_Groups},
      {"use_header_numbers", &Slrn_Use_Header_Numbers},
      {"use_localtime", &Slrn_Use_Localtime},
 #if SLRN_HAS_SPOILERS
@@ -745,6 +746,7 @@ Slrn_Str_Var_Type Slrn_Str_Variables [] = /*{{{*/
      {"top_status_line", &Slrn_Top_Status_Line},
      {"group_status_line", &Slrn_Group_Status_Line},
      {"scorefile", &Slrn_Score_File},
+     {"custom_sort_order", &Slrn_Sort_Order},
      {NULL, NULL}
 };
 
@@ -976,10 +978,10 @@ static Color_Handle_Type Auto_Color_Handles[] = /*{{{*/
 
 static char *Color_Names [16] =
 {
-   "black", "blue", "green", "cyan",
-     "red", "magenta", "brown", "lightgray",
-     "gray", "brightblue", "brightgreen", "brightcyan",
-     "brightred", "brightmagenta", "yellow", "white"
+   "black", "red", "green", "brown",
+     "blue", "magenta", "cyan", "lightgray",
+     "gray", "brightred", "brightgreen", "yellow",
+     "brightblue", "brightmagenta", "brightcyan", "white"
 };
 /*}}}*/
 
@@ -1029,10 +1031,14 @@ int slrn_set_object_color (char *name, char *fg, char *bg) /*{{{*/
 	ct++;
      }
 
-   if (0 == strncmp (name, "quotes", 6))
+   if (!strncmp (name, "quotes", 6))
      {
-	SLtt_set_color (QUOTE_COLOR + atoi (name+6), name, fg, bg);
-	return 0;
+	int level = atoi (name+6);
+	if ((0<=level) && (level<MAX_QUOTE_LEVELS))
+	  {
+	     SLtt_set_color (QUOTE_COLOR + level, name, fg, bg);
+	     return 0;
+	  }
      }
 
    slrn_error (_("%s is not a color object"), name);
@@ -1040,6 +1046,20 @@ int slrn_set_object_color (char *name, char *fg, char *bg) /*{{{*/
 }
 /*}}}*/
 
+#if defined(__unix__) && !defined(IBMPC_SYSTEM)
+static char *get_name_for_color (SLtt_Char_Type color, int want_bg) /*{{{*/
+{
+   /* 0xFF is the internal representation of "default" in S-Lang */
+   if (((color >> (want_bg ? 16 : 8)) & 0xFF) == 0xFF)
+     return "default";
+   if (color & SLTT_BOLD_MASK)
+     color |= 0x8 << 8;
+   if (color & SLTT_BLINK_MASK)
+     color |= 0x8 << 16;
+   return Color_Names[(color >> (want_bg ? 16 : 8)) & 0xF];
+}/*}}}*/
+#endif
+
 char *slrn_get_object_color (char *name, int want_bg) /*{{{*/
 {
 #if !defined(__unix__) || defined(IBMPC_SYSTEM)
@@ -1052,19 +1072,22 @@ char *slrn_get_object_color (char *name, int want_bg) /*{{{*/
      {
 	if (!strcmp (ct->name, name))
 	  {
-	     SLtt_Char_Type color;
-	     color = SLtt_get_color_object (ct->value);
-	     /* 0xFF is the internal representation of "default" in S-Lang */
-	     if (((color >> (want_bg ? 16 : 8)) & 0xFF) == 0xFF)
-	       return "default";
-	     if (color & SLTT_BOLD_MASK)
-	       color |= 0x8 << 8;
-	     if (color & SLTT_BLINK_MASK)
-	       color |= 0x8 << 16;
-	     return Color_Names[(color >> (want_bg ? 16 : 8)) & 0xF];
+	     SLtt_Char_Type color = SLtt_get_color_object (ct->value);
+	     return get_name_for_color (color, want_bg);
 	  }
 	ct++;
      }
+
+   if (!strncmp (name, "quotes", 6))
+     {
+	int level = atoi (name+6);
+	if ((0<=level) && (level<MAX_QUOTE_LEVELS))
+	  {
+	     SLtt_Char_Type color = SLtt_get_color_object (QUOTE_COLOR + level);
+	     return get_name_for_color (color, want_bg);
+	  }
+     }
+   
    slrn_error (_("%s is not a color object"), name);
 #endif
    return NULL;
