@@ -18,6 +18,7 @@
  59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 #include "config.h"
+#include "slrnfeat.h"
 
 #include <stdio.h>
 #include <string.h>
@@ -336,7 +337,7 @@ int slrn_dircat (char *dir, char *name, char *file, size_t n)
    len += 2;			       /* for / and \0 */
    if (len > n)
      {
-	slrn_error ("File name too long.");
+	slrn_error (_("File name too long."));
 	return -1;
      }
    
@@ -366,14 +367,14 @@ int slrn_dircat (char *dir, char *name, char *file, size_t n)
 
 static char *do_malloc_error (int do_error)
 {
-   if (do_error) slrn_error ("Memory allocation failure.");
+   if (do_error) slrn_error (_("Memory allocation failure."));
    return NULL;
 }
 
 char *slrn_safe_strmalloc (char *s) /*{{{*/
 {
    s = SLmake_string (s);
-   if (s == NULL) slrn_exit_error ("Out of memory.");
+   if (s == NULL) slrn_exit_error (_("Out of memory."));
    return s;
 }
 
@@ -429,7 +430,7 @@ char *slrn_safe_malloc (unsigned int len)
    s = slrn_malloc (len, 1, 0);
 
    if (s == NULL)
-     slrn_exit_error ("Out of memory");
+     slrn_exit_error (_("Out of memory"));
    
    return s;
 }
@@ -468,7 +469,7 @@ char *slrn_fix_regexp (char *pat) /*{{{*/
    len++;			       /* for \0 */
 
    if (len > sizeof(newpat))
-     slrn_exit_error ("Pattern too long for buffer");
+     slrn_exit_error (_("Pattern too long for buffer"));
      
    p = newpat;
 
@@ -526,7 +527,7 @@ char *slrn_spool_dircat (char *root, char *name, int map_dots)
    spool_group = SLmalloc (strlen (name) + len + 2);
    if (spool_group == NULL)
      {
-	slrn_exit_error ("Out of memory.");
+	slrn_exit_error (_("Out of memory."));
      }
 
    strcpy (spool_group, root); /* safe */
@@ -545,14 +546,14 @@ char *slrn_spool_dircat (char *root, char *name, int map_dots)
 #ifdef __CYGWIN__
    len = strlen (spool_group) + 9;
    if (NULL == (convdir = SLmalloc (len)))
-     slrn_exit_error ("Out of memory.");
+     slrn_exit_error (_("Out of memory."));
    if (0 == slrn_cygwin_convert_path (spool_group, convdir, len))
      {
 	slrn_free (spool_group);
 	spool_group = convdir;
      }
    else
-     slrn_error ("Cygwin conversion of %s failed.", spool_group);
+     slrn_error (_("Cygwin conversion of %s failed."), spool_group);
 #endif
 #if defined(IBMPC_SYSTEM)
    slrn_os2_convert_path (spool_group);
@@ -574,7 +575,7 @@ int slrn_delete_file (char *f) /*{{{*/
 int slrn_fclose (FILE *fp) /*{{{*/
 {
    if (0 == fclose (fp)) return 0;
-   slrn_error ("Error closing file.  File system full? (errno = %d)", errno);
+   slrn_error (_("Error closing file.  File system full? (errno = %d)"), errno);
    return -1;
 }
 
@@ -667,14 +668,14 @@ int slrn_copy_file (char *infile, char *outfile)
 
    if (NULL == (in = fopen (infile, "rb")))
      {
-	slrn_error ("Error opening %s", infile);
+	slrn_error (_("Error opening %s"), infile);
 	return -1;
      }
    
    if (NULL == (out = fopen (outfile, "wb")))
      {
 	fclose (in);
-	slrn_error ("Error opening %s", outfile);
+	slrn_error (_("Error opening %s"), outfile);
 	return -1;
      }
 
@@ -683,7 +684,7 @@ int slrn_copy_file (char *infile, char *outfile)
      {
 	if (EOF == putc (ch, out))
 	  {
-	     slrn_error ("Write Error: %s", outfile);
+	     slrn_error (_("Write Error: %s"), outfile);
 	     ret = -1;
 	     break;
 	  }
@@ -714,6 +715,7 @@ int slrn_move_file (char *infile, char *outfile)
    return 0;
 }
 
+
 unsigned int slrn_sleep (unsigned int len)
 {
 #ifdef __WIN32__
@@ -724,3 +726,30 @@ unsigned int slrn_sleep (unsigned int len)
    return sleep (len);
 #endif
 }
+
+/* Writes strings to stderr. If the _first_ character in a string is '\n',
+ * it gets replaced with '\r\n' on IBMPC_SYSTEM
+ * Note: We need this because gettext does not like '\r'. Sigh. */
+void slrn_va_stderr_strcat (const char *str, va_list args) /*{{{*/
+{
+   const char *p = str;
+   while (p != NULL)
+     {
+#ifdef IBMPC_SYSTEM
+	if (*p == '\n')
+	  fputc ('\r', stderr);
+#endif
+	fputs (p, stderr);
+	p = va_arg (args, const char *);
+     }
+}
+/*}}}*/
+
+void slrn_stderr_strcat (const char *str, ...) /*{{{*/
+{
+   va_list ag;
+   va_start (ag, str);
+   slrn_va_stderr_strcat (str, ag);
+   va_end (ag);
+}
+/*}}}*/
