@@ -3,7 +3,7 @@
  This file is part of SLRN.
 
  Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
- Copyright (c) 2001 Thomas Schultz <tststs@gmx.de>
+ Copyright (c) 2001, 2002 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -75,8 +75,6 @@ static Slrn_Header_Line_Type Xover_Headers [] = /*{{{*/
      {"Message-ID", 10, NULL, NULL},
 #define REFS_OFFSET     4
      {"References", 10, NULL, NULL},
-   /* Technically, bytes is in the overview file.  However, this value
-    * is not stored in slrn's article structure. */
 #define BYTES_OFFSET    5
      {"Bytes", 5, NULL, NULL},
 #define LINES_OFFSET    6
@@ -152,7 +150,11 @@ static void parse_headers (void) /*{{{*/
 					 (unsigned char *) addh->name)))
 	    {
 #if SLRN_HAS_MIME
-	       if (Slrn_Use_Mime & MIME_DISPLAY)
+	       if ((Slrn_Use_Mime & MIME_DISPLAY) &&
+		   slrn_case_strcmp ((unsigned char *)h,
+				     (unsigned char *)"Newsgroups") &&
+		   slrn_case_strcmp ((unsigned char *)h,
+				     (unsigned char *)"Followup-To"))
 		 slrn_rfc1522_decode_string (colon);
 #endif
 	       addh->value = colon;
@@ -188,7 +190,7 @@ static void parse_headers (void) /*{{{*/
 	       }
 	     
 	     if (ch == '\t') *h = ' ';
-
+	     
 	     h++;
 	  }
      }
@@ -376,6 +378,7 @@ void slrn_map_xover_to_header (Slrn_XOver_Type *xov, Slrn_Header_Type *h)
    h->date = xov->date;
    h->refs = xov->references;
    h->lines = xov->lines;
+   h->bytes = xov->bytes;
    h->xref = xov->xref;
    h->add_hdrs = xov->add_hdrs;
    
@@ -736,12 +739,15 @@ static int read_head_into_xover (int id, Slrn_XOver_Type *xov) /*{{{*/
 }
 /*}}}*/
 
+extern int Slrn_Prefer_Head;
+
 int slrn_open_xover (int min, int max) /*{{{*/
 {
    int id, status = -1;
    XOver_Done = 1;
 
-   if (Slrn_Server_Obj->sv_has_xover && !Suspend_XOver_For_Kill)
+   if (Slrn_Server_Obj->sv_has_xover && !Suspend_XOver_For_Kill &&
+       (Slrn_Prefer_Head != 2))
      {
 	rearrange_add_headers ();
 	status = Slrn_Server_Obj->sv_nntp_xover (min, max);
@@ -875,7 +881,8 @@ int slrn_read_xover (Slrn_XOver_Type *xov) /*{{{*/
    
    if (XOver_Done)
      {
-	if (!Slrn_Server_Obj->sv_has_xover || Suspend_XOver_For_Kill)
+	if (!Slrn_Server_Obj->sv_has_xover || Suspend_XOver_For_Kill ||
+	    (Slrn_Prefer_Head == 2))
 	  {
 	     Unretrieved_Headers = NULL;
 	     Current_Add_Header = NULL;
@@ -883,7 +890,8 @@ int slrn_read_xover (Slrn_XOver_Type *xov) /*{{{*/
 	return 0;
      }
 
-   if (Slrn_Server_Obj->sv_has_xover && !Suspend_XOver_For_Kill)
+   if (Slrn_Server_Obj->sv_has_xover && !Suspend_XOver_For_Kill &&
+       (Slrn_Prefer_Head != 2))
      {
 	while (1)
 	  {
@@ -942,7 +950,7 @@ int slrn_read_xover (Slrn_XOver_Type *xov) /*{{{*/
 		  break;
 	       }
 	     
-	     slrn_exit_error ("Server closed connection.  Cannot recover.");
+	     slrn_exit_error (_("Server closed connection.  Cannot recover."));
 	  }
 	
 	if ((status != OK_NEXT) || (id > XOver_Max))
@@ -988,11 +996,13 @@ int slrn_xover_for_msgid (char *msgid, Slrn_XOver_Type *xov) /*{{{*/
    if (OK_HEAD != status)
      {
 	if (ERR_FAULT == status)
-	  slrn_error ("Server does not provide this capability.");
+	  slrn_error (_("Server does not provide this capability."));
 	return -1;
      }
    
-   if (id == 0) id = -1;
+   /* As Leafnode sends article numbers from different groups,
+    * we always need to set this to -1. Sigh. */
+   /*if (id == 0)*/ id = -1;
    retval = read_head_into_xover (id, xov);
    
    return retval;
@@ -1002,7 +1012,8 @@ int slrn_xover_for_msgid (char *msgid, Slrn_XOver_Type *xov) /*{{{*/
 
 void slrn_open_all_add_xover (void) /*{{{*/
 {
-   if (!Slrn_Server_Obj->sv_has_xover || Suspend_XOver_For_Kill)
+   if (!Slrn_Server_Obj->sv_has_xover || Suspend_XOver_For_Kill ||
+       (Slrn_Prefer_Head == 2))
      Current_Add_Header = NULL;
    else
      Current_Add_Header = Additional_Headers;
