@@ -15,11 +15,12 @@
  more details.
 
  You should have received a copy of the GNU General Public License along
- with this program; if not, write to the Free Software Foundation, Inc., 675
- Mass Ave, Cambridge, MA 02139, USA. 
+ with this program; if not, write to the Free Software Foundation, Inc.,
+ 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 #ifndef _SLRN_XOVER_H
 #define _SLRN_XOVER_H
+
 /* In this structure, only subject_malloced will be malloced.  All other 
  * pointers point to a location in that space.  It is done this way because
  * art.c uses this convention and the pointer can just be passed to it.
@@ -28,8 +29,8 @@
 typedef struct
 {
    int id;
-   char *subject_malloced;
-   char *from;
+   char *subject_malloced; /* keep this separate from the rest */
+   char *from_malloced;
    char *date;
    char *message_id;
    char *references;
@@ -40,16 +41,15 @@ typedef struct
 }
 Slrn_XOver_Type;
 
-extern char *slrn_get_extra_xover_header (char *);
 extern void slrn_map_xover_to_header (Slrn_XOver_Type *, Slrn_Header_Type *);
-
-#ifndef SLRNPULL_CODE
-extern int slrn_read_overview_fmt (void);
-
 extern void slrn_free_additional_headers (Slrn_Header_Line_Type *);
 extern void slrn_clear_requested_headers (void);
 extern void slrn_request_additional_header (char *, int);
+extern char *slrn_extract_add_header (Slrn_Header_Type *, char *);
 
+extern int slrn_read_overview_fmt (void);
+
+#ifndef SLRNPULL_CODE
 extern int slrn_xover_for_msgid (char *, Slrn_XOver_Type *);
 extern int slrn_open_xover (int, int);
 extern int slrn_read_xover (Slrn_XOver_Type *);
